""" FeedListView.py

Module for displaying the feeds in the Feeds TreeView.
"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """
Straw is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Straw is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """

import copy
import locale
import pygtk
pygtk.require('2.0')
import gobject
import gtk
import pango
import FeedCategoryList
from FeedPropertiesDialog import FeedPropertiesDialog
import FeedList
import Feed
import Event
import dialogs
import Config
import PollManager
import MVP

class Column:
    NAME = 0
    UNREAD = 1
    BOLD = 2
    STATUS_FLAG = 3
    STATUS_PIXBUF = 4
    OBJECT = 5
    ALLOW_CHILDREN = 6

class FeedsView(MVP.WidgetView):
    popup_ui = """
    <ui>
    <popup name=\"feed_list_popup\">
        <menuitem action=\"refresh\"/>
        <menuitem action=\"mark_as_read\"/>
        <menuitem action=\"stop_refresh\"/>
        <menuitem action=\"remove\"/>
        <separator/>
        <menu name=\"Sort\" action=\"category-sort\">
            <menuitem action=\"ascending\"/>
            <menuitem action=\"descending\"/>
        </menu>
        <menuitem action=\"properties\"/>
    </popup>
    </ui>
    """
    def _initialize(self):
        self._popup = None
        self._widget.set_rules_hint(False)
        self._widget.set_search_column(Column.NAME)
        self._create_columns()
        selection = self._widget.get_selection()
        selection.connect("changed", self._feed_selection_changed, Column.OBJECT)
        self._widget.connect("button_press_event", self._on_button_press_event)
        self._widget.connect("popup-menu", self._on_popup_menu)
        self._create_popup()

    def _create_columns(self):
        # hide the expander. This should remove the extra space at the left
        # of the the treeview.
        expander = gtk.TreeViewColumn()
        expander.set_visible(False)
        self._widget.append_column(expander)
        self._widget.set_expander_column(expander)

        column = gtk.TreeViewColumn()
        status_renderer = gtk.CellRendererPixbuf()
        column.pack_start(status_renderer, False)
        column.set_attributes(status_renderer,
                              stock_id=Column.STATUS_PIXBUF,
                              visible=Column.STATUS_FLAG)
        unread_renderer = gtk.CellRendererText()
        column.pack_start(unread_renderer, False)
        column.set_attributes(unread_renderer,
                              text=Column.UNREAD, weight=Column.BOLD)
        column.set_title("_Subscriptions")
        title_renderer = gtk.CellRendererText()
        column.pack_end(title_renderer, False)
        column.set_attributes(title_renderer,
                              text=Column.NAME, weight=Column.BOLD)
        self._widget.append_column(column)
        return

    def _create_popup(self):
        actions = [
            ("refresh", gtk.STOCK_REFRESH, _("_Refresh"), None,
             _("Update this feed"), self._on_menu_poll_selected_activate),
            ("mark_as_read", gtk.STOCK_CLEAR, _("_Mark As Read"), None,
             _("Mark all items in this feed as read"), self._on_menu_mark_all_as_read_activate),
            ("stop_refresh", None, _("_Stop Refresh"), None,
             _("Stop updating this feed"), self._on_menu_stop_poll_selected_activate),
            ("remove", None, _("Remo_ve Feed"), None,
             _("Remove this feed from my subscription"), self._remove_selected_feed),
            ("category-sort",None,_("_Arrange Feeds"), None,
             _("Sort the current category")),
            ("ascending", gtk.STOCK_SORT_ASCENDING, _("Alpha_betical Order"), None,
             _("Sort in alphabetical order"), self._sort_ascending),
            ("descending", gtk.STOCK_SORT_DESCENDING, _("Re_verse Order"), None,
             _("Sort in reverse order"), self._sort_descending),
            ("properties", gtk.STOCK_INFO, _("_Information"), None,
             _("Feed-specific properties"), self._display_properties_feed)
            ]
        ag = gtk.ActionGroup('FeedListPopupActions')
        ag.add_actions(actions)
        uimanager = gtk.UIManager()
        uimanager.insert_action_group(ag,0)
        uimanager.add_ui_from_string(FeedsView.popup_ui)
        self._popup = uimanager.get_widget('/feed_list_popup')
        return

    def _model_set(self):
        self._widget.set_model(self._model)

    def get_selection(self):
        return self._widget.get_selection()

    def _on_popup_menu(self, treeview, *args):
        self._popup.popup(None, None, None, 0, 0)

    def _on_button_press_event(self, treeview, event):
        retval = 0
        if event.button == 3:
            x = int(event.x)
            y = int(event.y)
            time = gtk.get_current_event_time()
            path = treeview.get_path_at_pos(x, y)
            if path is None:
                return 1
            path, col, cellx, celly = path
            treeview.grab_focus()
            #treeview.set_cursor( path, col, 0)
            self._popup.popup(None, None, None, event.button, time)
            retval = 1
        return retval

    def _on_menu_poll_selected_activate(self, *args):
        self._presenter.poll_current_feed()

    def _on_menu_stop_poll_selected_activate(self, *args):
        self._presenter.stop_polling_current_feed()

    def _on_menu_mark_all_as_read_activate(self, *args):
        self._presenter.mark_current_feed_as_read()

    def _remove_selected_feed(self, *args):
        title = self._presenter.get_selected_feed().title
        response = dialogs.confirm_delete(_("Delete %s?") % title,
                                          _("Deleting %s will remove it from your subscription list.") % title)
        if (response == gtk.RESPONSE_OK):
            selection = self._widget.get_selection()
            self._presenter.remove_selected_feed()
        return

    def _sort_ascending(self, *args):
        self._presenter.sort_category()

    def _sort_descending(self, *args):
        self._presenter.sort_category(reverse=True)

    def _display_properties_feed(self, *args):
        self._presenter.show_feed_properties()
        return

    def _feed_selection_changed(self, selection, column):
        """
        Called when the current feed selection changed
        """
        model, rowiter = selection.get_selected()
        if not rowiter:
            return
        adapter = model.get_value(rowiter, column)
        if adapter:
            self._presenter.feed_selection_changed(adapter.feed)
        return

    def _on_feed_selection_treeview_row_expanded(self, widget,iter,path,*data):
        obj = self._model[path][Column.OBJECT]
        self._presenter.expand_row(obj)

    def _on_feed_selection_treeview_row_collapsed(self, widget,iter,path,*data):
        obj = self._model[path][Column.OBJECT]
        self._presenter.collapse_row(obj)

    def set_cursor(self, treeiter, col_id=None, edit=False):
        path = self._model.get_path(treeiter)
        if col_id:
            column = self._widget.get_column(col_id)
        else:
            column = None
        self._widget.set_cursor(path, column, edit)
        self._widget.scroll_to_cell(path, column)
        self._widget.grab_focus()
        return

    def queue_draw(self):
        self._widget.queue_draw()
        return


    def get_location(self):
        model, iter = self._widget.get_selection().get_selected()
        if iter is None:
            return (None, None)
        path = model.get_path(iter)
        return self.get_parent_with_path(FeedList.get_instance(), path)

class FeedsPresenter(MVP.BasicPresenter):
    def _initialize(self):
        self.initialize_slots(Event.FeedSelectionChangedSignal,
                              Event.FeedsEmptySignal)
        model = gtk.TreeStore(
            gobject.TYPE_STRING, gobject.TYPE_STRING,
            gobject.TYPE_INT, gobject.TYPE_BOOLEAN,
            gobject.TYPE_STRING, gobject.TYPE_PYOBJECT,
            gobject.TYPE_BOOLEAN)
        self.model = model
        self._init_signals()
        self._curr_feed = None
        self._curr_category = None
        return

    def _init_signals(self):
        flist = FeedList.get_instance()
        flist.signal_connect(Event.ItemReadSignal,
                             self._feed_item_read)
        flist.signal_connect(Event.AllItemsReadSignal,
                             self._feed_all_items_read)
        flist.signal_connect(Event.FeedsChangedSignal,
                             self._feeds_changed)
        flist.signal_connect(Event.FeedDetailChangedSignal,
                             self._feed_detail_changed)
        fclist = FeedCategoryList.get_instance()
        fclist.signal_connect(Event.FeedCategorySortedSignal,
                              self._feeds_sorted_cb)
        fclist.signal_connect(Event.FeedCategoryChangedSignal,
                              self._fcategory_changed_cb)

    def _sort_func(self, model, a, b):
        """
        Sorts the feeds lexically.

        From the gtk.TreeSortable.set_sort_func doc:

        The comparison callback should return -1 if the iter1 row should come before
        the iter2 row, 0 if the rows are equal, or 1 if the iter1 row should come
        after the iter2 row.
        """
        retval = 0
        fa = model.get_value(a, Column.OBJECT)
        fb = model.get_value(b, Column.OBJECT)

        if fa and fb:
            retval = locale.strcoll(fa.title, fb.title)
        elif fa is not None: retval = -1
        elif fb is not None: retval = 1
        return retval

    def poll_current_feed(self):
        config = Config.get_instance()
        poll = False
        if config.offline:
            response = dialogs.report_offline_status()
            if response == gtk.RESPONSE_OK:
                config.offline = not config.offline
                poll = True
        else:
            poll = True
        if poll:
            PollManager.get_instance().poll([self._curr_feed])
        return

    def get_selected_feed(self):
        return self._curr_feed

    def stop_polling_current_feed(self):
        self._curr_feed.router.stop_polling()

    def mark_current_feed_as_read(self):
        self._curr_feed.mark_all_read()

    def remove_selected_feed(self):
        selection = self._view.get_selection()
        model, rowiter = selection.get_selected()
        if rowiter:
            adapter = model.get_value(rowiter, Column.OBJECT)
            it = model.remove(rowiter)
            feedlist = FeedList.get_instance()
            idx = feedlist.index(adapter.feed)
            del feedlist[idx]
        return

    def sort_category(self, reverse=False):
        self._curr_category.sort()
        if reverse:
            self._curr_category.reverse()
        return

    def show_feed_properties(self):
        fpd = FeedPropertiesDialog.show_feed_properties(None, self._curr_feed)
        return

    def select_first_feed(self):
        treeiter = self._model.get_iter_first()
        if not treeiter or not self._model.iter_is_valid(treeiter):
            return False
        self._view.set_cursor(treeiter)
        return True

    def select_next_feed(self):
        """
        Scrolls to the next feed in the feed list
        """
        selection = self._view.get_selection()
        model, treeiter = selection.get_selected()
        if not treeiter:
            treeiter = model.get_iter_first()
        next_feed_iter = model.iter_next(treeiter)
        if not next_feed_iter or not self._model.iter_is_valid(next_feed_iter):
            return False
        self._view.set_cursor(next_feed_iter)
        return True

    def select_previous_feed(self):
        """
        Scrolls to the previous feeds in the feed list.
        """
        selection = self._view.get_selection()
        model, treeiter = selection.get_selected()
        if not treeiter:
            treeiter = model.get_iter_first()
        path = model.get_path(treeiter)
        # check if there's a feed in the path
        if not path:
            return False
        prev_path = path[-1]-1
        if prev_path < 0:
            # go to the last feed in the list
            prev_path = len(self._model) - 1
        self._view.set_cursor(self._model.get_iter(prev_path))
        return True

    def select_next_unread_feed(self):
        has_unread = False
        mark_treerow = 1
        treerow = self._model[0]
        selection = self._view.get_selection()
        srow = selection.get_selected()
        if srow:
            model, treeiter = srow
            nextiter = model.iter_next(treeiter)
            if nextiter:
                treerow = self._model[model.get_path(nextiter)]
        while(treerow):
            feedrow = treerow[Column.OBJECT]
            if feedrow.feed.n_items_unread:
                self._view.set_cursor(treerow.iter)
                has_unread = True
                break
            treerow = treerow.next
            if not treerow and mark_treerow:
                # should only do this once.
                mark_treerow = treerow
                treerow = self._model[0]
        return has_unread

    def feed_selection_changed(self, feed):
        """
        Called when the current feed selection was changed.

        This is called everytime a 'changed' signal in the treeview occurs
        """
        oldfeed = self._curr_feed
        if feed:
            self._curr_feed = feed
        else:
            self._curr_feed = None
        if oldfeed is not self._curr_feed:
            self.emit_signal(Event.FeedSelectionChangedSignal(self,oldfeed,self._curr_feed))
        return

    def display_feed(self, feed):
        # set_cursor will emit a 'changed' event in the treeview
        # and then feed_selection_changed above will be called.
        path = self._curr_category.index_feed(feed)
        treeiter = self._model.get_iter(path)
        self._view.set_cursor(treeiter, Column.NAME)

    def display_category_feeds(self, category):
        self._curr_category = category
        feeds = self._curr_category.feeds
        self._model.foreach(self._disconnect)
        self._model.clear()
        curr_feed_iter = self._display_feeds(feeds)
        if curr_feed_iter:
            self._view.set_cursor(curr_feed_iter)
        else:
            it = self._model.get_iter_first()
            if it:
                self._view.set_cursor(it)
            else:
                self.emit_signal(Event.FeedsEmptySignal(self))
        return

    def _display_feeds(self, feeds, parent=None):
        def _connect_adapter(adapter, feedindex):
            adapter.signal_connect(Event.ItemsAddedSignal,
                                   self._adapter_updated_handler, feedindex)
            adapter.signal_connect(Event.FeedPolledSignal,
                                   self._adapter_updated_handler, feedindex)
            adapter.signal_connect(Event.FeedStatusChangedSignal,
                                   self._adapter_updated_handler, feedindex)
            adapter.signal_connect(Event.ItemReadSignal,
                                   self._adapter_updated_handler, feedindex)
            adapter.signal_connect(Event.AllItemsReadSignal,
                                   self._adapter_updated_handler, feedindex)
        curr_feed_iter = None
        for f in feeds:
            adapter = create_adapter(f)
            rowiter = self._model.append(parent)
            self._model.set(rowiter,
                            Column.NAME, adapter.title,
                            Column.OBJECT, adapter)

            idx = self._curr_category.index_feed(adapter.feed)
            _connect_adapter(adapter, idx) # we need this to get the rest of the data
            new_string, new = adapter.unread
            weight = (pango.WEIGHT_NORMAL, pango.WEIGHT_BOLD)[new > 0]
            status, pixbuf = adapter.status_icon
            self._model.set(rowiter,
                            Column.UNREAD, new_string,
                            Column.BOLD, weight,
                            Column.STATUS_FLAG, status,
                            Column.STATUS_PIXBUF, pixbuf,
                            Column.ALLOW_CHILDREN, False)
            if adapter.feed is self._curr_feed:
                curr_feed_iter = rowiter
        return curr_feed_iter

    def display_empty_category(self):
        self._model.foreach(self._disconnect)
        self._model.clear()
        return

    def _disconnect(self, model, path, iter, user_data=None):
        ob = model[path][Column.OBJECT]
        self._disconnect_adapter(ob)
        if not len(model):
            return True
        return False

    def _disconnect_adapter(self, adapter):
        adapter.disconnect()
        adapter.signal_disconnect(Event.ItemsAddedSignal,
                                  self._adapter_updated_handler)
        adapter.signal_disconnect(Event.FeedPolledSignal,
                                  self._adapter_updated_handler)
        adapter.signal_disconnect(Event.FeedStatusChangedSignal,
                                  self._adapter_updated_handler)
        del adapter

    def _adapter_updated_handler(self, signal, feed_index):
        self._update_adapter_view(signal.sender, feed_index)

    def _update_adapter_view(self, adapter, feed_index):
        new = adapter.unread
        row = self._model[feed_index]
        row[Column.NAME] = adapter.title
        row[Column.UNREAD] = new[0]
        row[Column.BOLD] = (pango.WEIGHT_NORMAL, pango.WEIGHT_BOLD)[new[1] > 0]
        row[Column.OBJECT] = adapter
        status, pixbuf = adapter.status_icon
        row[Column.STATUS_FLAG] = status
        if pixbuf:
            row[Column.STATUS_PIXBUF] = pixbuf
        self._view.queue_draw()

    def _feeds_changed(self, signal):
        self._feed_view_update(signal.feed)

    def _feed_detail_changed(self, signal):
        self._feed_view_update(signal.sender)

    def _feed_item_read(self, signal):
        path = (0,)
        selection = self._view.get_selection()
        selected_row = selection.get_selected()
        if selected_row:
            model, treeiter = selected_row
            path = model.get_path(treeiter)
        treerow = self._model[path]
        adapter = treerow[Column.OBJECT]
        self._update_adapter_view(adapter, path)

    def _feed_all_items_read(self, signal):
        pass

    def _feed_view_update(self, feed):
        for index, f in enumerate(self._model):
            adapter = self._model[index][Column.OBJECT]
            if adapter.feed is feed:
                self._update_adapter_view(adapter, index)
                break
        return

    def _feeds_sorted_cb(self, signal):
        self.model.set_sort_func(Column.NAME, self._sort_func)
        if not signal.descending:
            self.model.set_sort_column_id(Column.NAME, gtk.SORT_ASCENDING)
        else:
            self.model.set_sort_column_id(Column.NAME, gtk.SORT_DESCENDING)
        self.model.sort_column_changed()
        return

    def _fcategory_changed_cb(self,signal):
        if signal.sender is self._curr_category:
            self._curr_category = signal.sender
            self.display_category_feeds(self._curr_category)
        return

    def expand_row(self, obj):
        obj.open = True

    def collapse_row(self, obj):
        obj.open = False

    def move_feed(self, sidx, tidx):
        self._curr_category.move_feed(sidx, tidx)
        return

class DisplayAdapter(object, Event.SignalEmitter):
    """
    View adapter for feeds and categories
    """
    def __init__(self, ob):
        self._ob = ob
        Event.SignalEmitter.__init__(self)
        self.initialize_slots(Event.ItemReadSignal,
                              Event.ItemsAddedSignal,
                              Event.AllItemsReadSignal,
                              Event.FeedPolledSignal,
                              Event.FeedStatusChangedSignal)

    def equals(self, ob):
        return self._ob is ob

class FeedDisplayAdapter(DisplayAdapter):
    """Adapter for displaying Feed objects in the tree"""
    def __init__(self, ob):
        DisplayAdapter.__init__(self, ob)
        ob.signal_connect(Event.ItemReadSignal, self.resend_signal)
        ob.signal_connect(Event.ItemsAddedSignal, self.resend_signal)
        ob.signal_connect(Event.AllItemsReadSignal, self.resend_signal)
        ob.signal_connect(Event.FeedPolledSignal, self.resend_signal)
        ob.signal_connect(Event.FeedStatusChangedSignal, self.resend_signal)

    def disconnect(self):
        self._ob.signal_disconnect(
            Event.ItemReadSignal, self.resend_signal)
        self._ob.signal_disconnect(
            Event.ItemsAddedSignal, self.resend_signal)
        self._ob.signal_disconnect(
            Event.AllItemsReadSignal, self.resend_signal)
        self._ob.signal_disconnect(
            Event.FeedPolledSignal, self.resend_signal)
        self._ob.signal_disconnect(
            Event.FeedStatusChangedSignal, self.resend_signal)

    def resend_signal(self, signal):
        new = copy.copy(signal)
        new.sender = self
        self.emit_signal(new)

    @property
    def title(self):
        return self._ob.title

    @property
    def unread(self):
        nu = self._ob.n_items_unread
        if nu != 0:
            return ("%s" % nu, nu)
        else:
            return ("", nu)

    @property
    def status_icon(self):
        if self._ob.process_status is not Feed.Feed.STATUS_IDLE:
            return (1, gtk.STOCK_EXECUTE)
        elif self._ob.error:
            return (1, gtk.STOCK_DIALOG_ERROR)
        return (0, None)

    @property
    def feed(self):
        return self._ob

    contents = property(lambda x: None)
    open = property(lambda x: None)

def create_adapter(ob):
    if isinstance(ob, Feed.Feed):
        return FeedDisplayAdapter(ob)




""" Event.py

Module for dispatching Straw-related events.
"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """ GNU General Public License

This program is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """

import traceback
import sys
import error

_base_cache = dict()

def find_bases(klass):
    def _find_bases(klass, d = None):
        if d is None:
            d = dict()
        for b in klass.__bases__:
            if issubclass(b, BaseSignal):
                d[b] = 1
                _find_bases(b, d)
        return tuple(d.keys())
    bases = _base_cache.get(klass, None)
    if bases is not None:
        return bases
    bases = _find_bases(klass)
    _base_cache[klass] = bases
    return bases

class BaseSignal:
    def __init__(self, sender):
        self.sender = sender
        return

class SignalEmitter:
    recorder = None

    """Base class for signal emitters"""
    def __init__(self):
        self._slots = dict()

    def initialize_slots(self, *signals):
        """Initialize signal slots.
        signals should be a list of signals this emitter is going to send."""
        for s in signals:
            self._slots[s] = dict()
            bs = find_bases(s)
            for k in bs:
                self._slots[k] = dict()

    def signal_connect(self, signal, handler, data=None):
        """Connect a handler to a signal in this emitter, and associate
        data with the callback. This will override previous bindings of
        signal to handler."""
        self._slots[signal][handler] = data
        return

    def signal_disconnect(self, signal, handler):
        """Disconnect handler from signal."""
        self._slots[signal][handler] = None
        del self._slots[signal][handler]
        return

    def emit_signal(self, signal):
        """Emit a signal object. Signal should be an instance of BaseSignal or
        a subclass."""
        sclasses = find_bases(signal.__class__)
        for sc in sclasses + (signal.__class__,):
            for handler, data in self._slots.get(sc, {}).items():
                if data is not None:
                    args = (signal, data)
                else:
                    args = (signal,)
                try:
                    #if self.recorder is not None:
                    #    self.recorder.record(sc, handler)
                    apply(handler, args)
                except:
                    error.log_exc("Caught an exception when trying to "
                                  "call a signal handler for "
                                  "signal %s\n" %
                                  str(signal))
                    traceback.print_exc()

    """
    def start_recording(klass, recorder):
        klass.recorder = recorder
    start_recording = classmethod(start_recording)

    def stop_recording(klass):
        if klass.recorder is not None:
            klass.recorder.stop()
            klass.recorder = None
    stop_recording = classmethod(stop_recording)
    """

# Feed creation and feed list management events

class FeedsChangedSignal(BaseSignal):
    def __init__(self, sender, feed=None):
        BaseSignal.__init__(self, sender)
        self.feed = feed

class FeedsEmptySignal(BaseSignal):
    pass

class FeedsImportedSignal(FeedsChangedSignal):
    """
    A signal for importing huge amount of feeds from OPML.
    """
    def __init__(self, sender, feeds,
                 parent=None, from_sub=False):
        FeedsChangedSignal.__init__(self, sender)
        self.feeds = feeds
        self.category = parent
        self.from_sub = from_sub

class FeedDeletedSignal(FeedsChangedSignal):
    def __init__(self, sender, feed):
        FeedsChangedSignal.__init__(self, sender, feed)
        self.feed = feed

class FeedCreatedSignal(FeedsChangedSignal):
    def __init__(self, sender, feed, category=None, index=None):
        FeedsChangedSignal.__init__(self, sender)
        self.feed = feed
        self.category = category
        self.index = index

# Feed detail Specific
class FeedDetailChangedSignal(BaseSignal):
    def __init__(self, sender):
        BaseSignal.__init__(self, sender)

class ItemOrderChangedSignal(FeedDetailChangedSignal):
    pass

class FeedPollFrequencyChangedSignal(FeedDetailChangedSignal):
    def __init__(self, sender):
        BaseSignal.__init__(self, sender)

class FeedNumOfItemsStoredChangedSignal(FeedDetailChangedSignal):
    def __init__(self, sender):
        BaseSignal.__init__(self, sender)

class FeedErrorStatusChangedSignal(FeedDetailChangedSignal):
    pass

class FeedInfoUpdatedSignal(FeedDetailChangedSignal):
    """
    Feed information like title, description, copyright, etc...
    """
    def __init__(self, sender):
        BaseSignal.__init__(self, sender)

class FeedUnreadChangedSignal(FeedDetailChangedSignal):
    def __init__(self, sender):
        BaseSignal.__init__(self, sender)


# Feed category specific

class FeedCategoriesChangedSignal(BaseSignal):
    pass

class FeedCategoryListChangedSignal(FeedCategoriesChangedSignal):
    pass

class FeedCategoryListLoadedSignal(FeedCategoriesChangedSignal):
    pass

class FeedCategoryAddedSignal(FeedCategoryListChangedSignal):
    def __init__(self, sender, category):
        FeedCategoryListChangedSignal.__init__(self, sender)
        self.category = category

class FeedCategoryRemovedSignal(FeedCategoryListChangedSignal):
    def __init__(self, sender, category):
        FeedCategoryListChangedSignal.__init__(self, sender)
        self.category = category

class FeedCategoryChangedSignal(FeedCategoriesChangedSignal):
    """ Called when a Feed Category was modified (i.e. feed added, feed
    removed, category title changed, etc...
    """
    def __init__(self, sender, feed=None):
        FeedCategoriesChangedSignal.__init__(self, sender)
        self.feed = feed

class FeedCategorySortedSignal(FeedCategoryChangedSignal):
    """ Called when a Feed Category has been sorted. """
    def __init__(self, sender, reverse=False):
        FeedCategoryChangedSignal.__init__(self, sender)
        self.descending = reverse

class SubscriptionContentsUpdatedSignal(BaseSignal):
    pass

# Item specific
class ItemsAddedSignal(BaseSignal):
    def __init__(self, sender, items):
        BaseSignal.__init__(self, sender)
        self.items = items

class ItemDeletedSignal(BaseSignal):
    def __init__(self, sender, item):
        BaseSignal.__init__(self, sender)
        self.item = item

class NewItemsSignal(BaseSignal):
    def __init__(self, sender, items):
        BaseSignal.__init__(self, sender)
        self.items = items

class ItemStickySignal(BaseSignal):
    def __init__(self, sender, item=None):
        BaseSignal.__init__(self, sender)
        if item is None:
            item = sender
        self.item = item

class AllItemsReadSignal(BaseSignal):
    def __init__(self, sender, changed):
        BaseSignal.__init__(self, sender)
        self.changed = changed


# Other events ...

class FeedLastPollChangedSignal(BaseSignal):
    pass

class PollChangedSignal(BaseSignal):
    def __init__(self, sender):
        BaseSignal.__init__(self, sender)

class FeedPolledSignal(BaseSignal):
    pass

class FeedStatusChangedSignal(BaseSignal):
    """
    Provides a signal for denoting that the polling process
    has begun or finished.
    """
    pass

class ImageUpdatedSignal(BaseSignal):
    def __init__(self, sender, url, data):
        BaseSignal.__init__(self, sender)
        self.url = url
        self.data = data

class NumberOfItemsStoredChangedSignal(BaseSignal):
    pass

class PollFrequencyChangedSignal(PollChangedSignal):
    """ Global poll frequency """
    def __init__(self, sender, value):
        PollChangedSignal.__init__(self, sender)
        self.value = value

class PollingStoppedSignal(BaseSignal):
    pass

class FeedSelectionChangedSignal(BaseSignal):
    """
    Emitted when a feed has been selected in the feeds tree view.
    """
    def __init__(self, sender, old, new):
        BaseSignal.__init__(self, sender)
        self.old = old
        self.new = new

class ItemSelectionChangedSignal(BaseSignal):
    """ Called when an item has been selected in the item list.
    Normally this should display the content of the item
    """
    def __init__(self, sender, item):
        BaseSignal.__init__(self, sender)
        self.item = item

class ItemReadSignal(BaseSignal):
    """ Called when an item's 'seen' property has been changed
    (i.e. marked as read)
    """
    def __init__(self, sender, item=None):
        BaseSignal.__init__(self, sender)
        if item is None:
            item = sender
        self.item = item

class CategorySelectionChangedSignal(BaseSignal):
    """
    Emitted when a the user changes categories
    """
    def __init__(self, sender, newcat):
        BaseSignal.__init__(self, sender)
        self.current = newcat

class OfflineModeChangedSignal(BaseSignal):
    pass

class RefreshFeedDisplaySignal(BaseSignal):
    pass

class StatusDisplaySignal(BaseSignal):
    pass

class FindInterruptSignal(BaseSignal):
    pass


if __name__ == '__main__':
    print find_bases(ItemReadSignal)



""" MVP.py

Module that provides a Model-View-Presenter framework for use with Straw
"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """
Straw is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Straw is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """


import pygtk
pygtk.require('2.0')
from gtk import glade
import Event


__doc__ = "A MVP framework. Contains base classes for views and presenters."

class View(object):
    """Base class for widget-based view objects."""
    def __init__(self, widget, model = None, presenter = None):
        self._widget = widget
        self._model = model
        self._presenter = presenter

        if model is not None:
            self._model_set()
        if presenter is not None:
            self._presenter_set()

        self._initialize()
        self._connect_signals(widget)

    def _connect_signals(self, widget):
        raise NotImplemented

    def _initialize(self):
        """Called from the constructor after the instance variables have
        been set and the corresponding initialization functions have been
        called, but before connecting gtk signals."""
        pass

    def _model_set(self):
        """Called when the model is set."""
        pass

    def _presenter_set(self):
        """Called when the presenter is set."""
        pass

    def get_model(self):
        return self._model

    def set_model(self, model):
        self._model = model
        if model:
            # don't call model_set if model is None. This prevents objects
            # relying on model_set to fail (or gtk throwing a warning when
            # model is not a gtk.TreeModel).
            self._model_set()
    model = property(get_model, set_model, None, doc="The model object.")

    def get_presenter(self):
        return self._presenter

    def set_presenter(self, presenter):
        self._presenter = presenter
        self._presenter_set()
    presenter = property(get_presenter, set_presenter, None,
                         "The presenter object.")

class WidgetView(View):
    def __init__(self, widget, model=None, presenter=None):
        View.__init__(self, widget, model, presenter)

    def _connect_signals(self, widget):
        xml = glade.get_widget_tree(widget)
        nameFuncMap = {}
        for key in dir(self.__class__):
            if key[:4] == '_on_':
                nameFuncMap[key[1:]] = getattr(self, key)
        xml.signal_autoconnect(nameFuncMap)

class GladeView(View):
    def __init__(self, widget, model=None, presenter=None):
        View.__init__(self, widget, model, presenter)

    def _connect_signals(self, widget):
        nameFuncMap = {}
        for key in dir(self.__class__):
            if key[:4] == '_on_':
                nameFuncMap[key[1:]] = getattr(self, key)
        widget.signal_autoconnect(nameFuncMap)

class BasicPresenter(object, Event.SignalEmitter):
    """Base class for basic (non-composite) presenters."""
    def __init__(self, model = None, view = None):
        Event.SignalEmitter.__init__(self)

        self._model = model
        self._view = view

        if model is not None:
            self._model_set()
        if view is not None:
            view.model = model
            view.presenter = self
            self._view_set()

        self._initialize()

    def _initialize(self):
        pass

    def _model_set(self):
        pass

    def _view_set(self):
        pass

    def get_view(self):
        return self._view

    def set_view(self, view):
        self._view = view
        view.presenter = self
        view.model = self._model
        self._view_set()

    view = property(get_view, set_view, None,
                    """The view component. Setting this als sets the view
                    object's presenter and model attributes.""")

    def get_model(self):
        return self._model

    def set_model(self, model):
        self._model = model
        if self._view is not None:
            self._view.model = model
        self._model_set()

    model = property(get_model, set_model, None,
                     """The model component. Setting this also sets the view
                     object's (if defined) model attribute.""")



""" LookupManager.py

Module that wraps the DNS resolver.
"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """ GNU General Public License

This program is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """


import time
import socket
import random
import error
import Config
try:
    import adns, ADNS
    _have_adns = True
except ImportError:
    error.log("No ADNS library found, using synchronous name lookups.")
    _have_adns = False

MAX_TIMES = 3

class NameFormatException(Exception):
    pass

class CBWrapper:
    def __init__(self, name, callback, data, times = 0):
        self.name = name
        self.data = data
        self.callback = callback
        self.times = 0

    def cb(self, ip):
        self.callback(self.name, ip, self.data)

# This is the threaded version: lookups are done in a thread,
# LookupMananger communicates via two Queue objects with it.
import Queue
import threading

class LookupThread(threading.Thread):
    def __init__(self, namequeue, ipqueue, group=None, target=None,
                 name=None, *args, **kwargs):
        threading.Thread.__init__(self, group, target, name, args, kwargs)
        self._namequeue = namequeue
        self._ipqueue = ipqueue
        self.setDaemon(True)

    def run(self):
        while True:
            name, cbw = self._namequeue.get(True)
            try:
                ip = socket.gethostbyname(name)
            except socket.error:
                ip = None
            self._ipqueue.put((cbw, ip))

class ThreadLookupManager:
    NameFormatException = NameFormatException
    namecache = {}

    def __init__(self, mintime=86400, maxtime=259200):
        self.mintime = mintime
        self.maxtime = maxtime
        self._namequeue = Queue.Queue(0)
        self._ipqueue = Queue.Queue(0)
        self._queryengine = LookupThread(
            namequeue=self._namequeue, ipqueue=self._ipqueue)
        self._queryengine.start()

    def _lookup(self, name, cbw):
        now = time.time()
        ip = self.namecache.get(name, None)
        expiretime = 0
        if ip is not None:
            ip, expiretime = ip
            if now < expiretime:
                cbw.cb(ip)
                return
        self._namequeue.put((name, cbw))

    def lookup(self, name, callback, data=None):
        cbw = CBWrapper(name, callback, data)
        self._lookup(name, cbw)

    def poll(self, timeout=0.1):
        t1 = time.time()
        while True:
            try:
                cbw, ip = self._ipqueue.get_nowait()
                cbw.cb(ip)
            except Queue.Empty:
                break
            if time.time() - t1 > timeout:
                break
            
# The ADNS version
if _have_adns:
    class MyQE(ADNS.QueryEngine):
        def lookup_a_record(self, name, callback, extra):
            self.submit(name, adns.rr.A, callback = callback, extra = extra)

    class ADNSLookupManager:
        namecache = {}
        queryengine = MyQE()

        def __init__(self, mintime=86400, maxtime=259200):
            self.mintime = mintime
            self.maxtime = maxtime

        def _lookup(self, name, callback, cbw):
            try:
                name = name.encode('ascii')
            except UnicodeError:
                raise NameFormatException, "Host names must be ASCII"
            now = time.time()
            ip = self.namecache.get(name, None)
            expiretime = 0
            if ip is not None:
                ip, expiretime = ip
                if now < expiretime:
                    cbw.cb(ip)
                    return
            self.queryengine.lookup_a_record(name, callback, cbw)

        def lookup(self, name, callback, data=None):
            cbw = CBWrapper(name, callback, data)
            self._lookup(name, self.adns_callback, cbw)

        def adns_callback(self, answer, qname, rr, flags, cbwrapper):
            now = int(time.time())
            ips = answer[3]
            ip = None
            if answer[1] and not ips:
                # we got a cname even though we asked for an a record
                self._lookup(answer[1], self.adns_callback, cbwrapper)
                return
            elif ips:
                ip = ips[0]
                self.namecache[qname] = (ip, now+random.randint(self.mintime,
                                                                self.maxtime))
            else:
                # try again
                if cbwrapper.times < MAX_TIMES:
                    cbwrapper.times += 1
                    self._lookup(cbwrapper.name, self.adns_callback, cbwrapper)
                    return
            cbwrapper.cb(ip)

        def poll(self, timeout=0.1):
            self.queryengine.run(timeout)

# Blocking version: use normal lookups, no threads
class BlockingLookupManager:
    NameFormatException = NameFormatException
    namecache = {}

    def __init__(self, mintime=86400, maxtime=259200):
        self.mintime = mintime
        self.maxtime = maxtime

    def _lookup(self, name):
        now = time.time()
        ip = self.namecache.get(name, None)
        expiretime = 0
        if ip is not None:
            ip, expiretime = ip
            if now < expiretime:
                return ip
        try:
            ip = socket.gethostbyname(name)
        except socket.error:
            ip = None
        now = int(time.time())
        self.namecache[name] = (ip, now + random.randint(self.mintime,
                                                         self.maxtime))
        return ip

    def lookup(self, name, callback, data=None):
        try:
            name = name.encode('ascii')
        except UnicodeError:
            raise NameFormatException, "Host names must be ASCII"
        ip = self._lookup(name)
        callback(name, ip, data)

    def poll(self, timeout=0.1):
        pass

lookupmanager_instance = None

def get_instance():
    global lookupmanager_instance
    if lookupmanager_instance is None:
        if Config.get_instance().use_threads:
            #error.log("using threaded lookups")
            lookupmanager_instance = ThreadLookupManager()
        elif _have_adns:
            #error.log("using ADNS for lookups")
            lookupmanager_instance = ADNSLookupManager()
        else:
            #error.log("using blocking lookups")
            lookupmanager_instance = BlockingLookupManager()
    return lookupmanager_instance




""" URLFetch.py

Module for retrieving data from a URL.

"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc"
__license__ = """
Straw is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Straw is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """



import httplib_async
import asyncore
import urlparse
import base64
import gzip
try:
    from cStringIO import StringIO
except:
    from StringIO import StringIO
import error
import time
import NetworkConstants
import Config
import LookupManager
import constants

def is_ip(host):
    parts = host.split(".")
    if len(parts) != 4:
        return 0
    for p in parts:
        try:
            np = int(p)
        except (ValueError, TypeError):
            return 0
        if not 0 <= np < 256:
            return 0
    return 1

class RequestSchemeException(Exception):
    def __init__(self, scheme):
        Exception.__init__(self)
        self.scheme = scheme
        self.args = {}

class Request:
    def __init__(self, host = None, port = None, path = None, ip = None,
                 headers = None, user = None, password = None, priority = None,
                 consumer = None, uri = None):
        self.host = host
        self.port = port
        self.path = path
        self.ip = ip
        self.headers = headers
        self.user = user
        self.password = password
        self.priority = priority
        self.consumer = consumer
        self.uri = uri

class Caller:
    def __init__(self, f):
        self._f = f

    def __call__(self):
        if not hasattr(self, "_f"): return
        try:
            self._f()
        finally:
            del self._f

class ConnectionManager:
    def __init__(self):
        self._queue = []

    def request(self, uri, consumer, headers={}, user=None, password=None, priority=NetworkConstants.PRIORITY_DEFAULT):
        uri = uri.strip()
        scheme, host, path, params, query, fragment = urlparse.urlparse(uri)
        if scheme != "http":
            raise RequestSchemeException(scheme)
        try:
            host, port = host.split(":", 1)
            port = int(port)
        except (TypeError, ValueError):
            port = 80 # default port

        if not path:
            path = "/"
        if params:
            path = path + ";" + params
        if query:
            path = path + "?" + query
        req = Request(host = host, port = port, path = path, headers = headers,
                      user = user, password = password, priority = priority,
                      consumer = consumer, uri = uri)
        pc = Config.get_instance().proxy_config
        if pc.use:
            # we don't set ConsumerAdapter's proxy here because we might not
            # yet know its ip. However, we can set the Proxy-Authorization
            # line
            if pc.use_authentication:
                req.headers['Proxy-Authorization'] = ('Basic %s' % base64.encodestring('%s:%s' % (pc.user, pc.password)).strip())
        adapter = ConsumerAdapter(req)

        # no lookup necessary if host is an ip address.
        # if we are using a proxy, let it handle the lookup.
        if is_ip(host):
            adapter.request.ip = host
            self._queue_request(adapter)
        elif pc.use:
            self._queue_request(adapter)
        else:
            try:
                LookupManager.get_instance().lookup(
                    host, self._request_resolved, adapter)
            except LookupManager.NameFormatException, e:
                adapter.http_failed(e)
        # don't return the categoryadapter object, because nothing in
        # it except stop() should be called from other parts of Straw
        return Caller(adapter.stop)
    
    def _queue_request(self, adapter):
        i = 0
        while i < len(self._queue):
            if self._queue[i].request.priority > adapter.request.priority:
                self._queue.insert(i, adapter)
                return
            i += 1
            # from outside while
        self._queue.append(adapter)

    def _request_resolved(self, host, ip, adapter):
        if ip is not None and ip != "":
            adapter.request.ip = ip
            self._queue_request(adapter)
        else:
            adapter.http_failed(_("Host name lookup failed"))

    def poll(self, timeout=0.1):
        proxy_config = Config.get_instance().proxy_config
        lookup_manager = LookupManager.get_instance()
        # activate up to MAX_CONNECTIONS channels
        while self._queue and len(asyncore.socket_map) < NetworkConstants.MAX_CONNECTIONS and not proxy_config.is_waiting:
            adapter = self._queue.pop(0)
            # has the user switched off the proxy after this request was queued
            if (not proxy_config.use) and (not adapter.request.ip):
                lookup_manager.lookup(
                    adapter.request.host, self._request_resolved, adapter)
            else:
                if proxy_config.use:
                    adapter.set_proxy((proxy_config.ip, proxy_config.port))
                adapter.start()
        # keep the network running
        now = time.time()
        lookup_manager.poll(timeout)
        timeout -= (time.time() - now)
        if timeout > 0.0:
            asyncore.poll(timeout)
        # time out stuck consumers
        self.time_out_consumers()
        # return non-zero if we should keep polling
        return len(self._queue) or len(asyncore.socket_map)

    def time_out_consumers(self):
        now = time.time()
        for obj in asyncore.socket_map.values():
            pc = obj.consumer
            if now - pc.start_time > NetworkConstants.MAX_DOWNLOAD_TIME:
                pc.time_exceeded()

connection_manager = ConnectionManager()

class ConsumerAdapter(object):
    CREATED = 0
    STARTED = 1
    CLOSED = 2

    # TODO: we do nothing with the proxy?
    def __init__(self, req):
        self.connection = httplib_async.HTTPConnection_async(req.host, req.ip, req.port, self)
        self.consumer = req.consumer
        self._data = ""
        self.header = None
        self.status = None
        self._request = req
        self.finished = False
        self.start_time = None
        self.state = self.CREATED
        if req.user and req.password:
            req.headers['Authorization'] = 'Basic %s' % base64.encodestring('%s:%s' % (req.user, req.password)).strip()
        req.headers['Accept-encoding'] = 'gzip'
        req.headers['User-agent'] = 'Straw/%s' % constants.VERSION

    # interface used by ConnectionManager
    def start(self):
        if self.state != self.CREATED:
            return
        self.state = self.STARTED
        self.start_time = time.time()
        try:
            self.connection.request(
                "GET", self._request.path, self._request.headers)
            self.connection.execute()
        except Exception, ex:
            self._send_failed(ex)
            self._close_connection()
            self.state = self.CLOSED

    def set_proxy(self, proxy):
        self.connection.set_proxy(proxy)

    def time_exceeded(self):
        if self.state != self.STARTED:
            return
        self.state = self.CLOSED
        self._close_connection()
        self._send_failed(_("Maximum download time exceeded"))

    def get_request(self):
        return self._request
    request = property(get_request)

    def stop(self):
        if self.state != self.CLOSED:
            self.state = self.CLOSED
            self._close_connection()
            self.consumer.operation_stopped()

    # the following methods are the interface HTTPConnection_async uses
    def http_failed(self, exception):
        """Called by HTTPConnection_async when connection failed with
        exception. Also used by ConnectionManager if host name lookup failed.
        """
        self._send_failed(exception)
        self._close_connection()
        self.state = self.CLOSED

    def http_header(self, status, header):
        """Called by HTTPConnection_async with status and header"""
        self.status = status
        self.header = header
        if header.getheader('content-length') == '0':
            self._send_results()
            self._close_connection()
            self.state = self.CLOSED

    def http_redirect(self, location, permanent = 0):
        """Called by HTTPConnection_async with the new location in case of 301 or 302"""
        assert type(location) == type(''), "Invalid redirect"
        if urlparse.urlparse(location)[0] != 'http':
            location = urlparse.urljoin(self.request.uri, location)
        if permanent:
            self.consumer.http_permanent_redirect(location)

        connection_manager.request(
            location, self.consumer, self.request.headers,
            self.request.user, self.request.password)
        self._close_connection()
        self.state = self.CLOSED

    def feed(self, data):
        """Called by HTTPConnection_async with (part of the) data, after http_header"""
        self._data += data
        datalength = len(self._data)
        cl = self.header.getheader('content-length')
        if cl is not None and datalength >= int(cl):
        #if datalength >= int(self.header.getheader('content-length', 0)):
            self._close_connection()
            self._send_results()
            self.state = self.CLOSED
        elif datalength >= NetworkConstants.MAX_DOWNLOAD_SIZE:
            self._close_connection()
            self._send_failed(_("Maximum download file size exceeded"))
            self.state = self.CLOSED

    def http_close(self):
        """Called by HTTPConsumer_async when the connection is closed"""
        if self.header and (len(self._data) <
                            int(self.header.getheader('content-length', 0))):
            msg = _("Feed is empty.")
            self._send_failed(msg)
        else:
            self._send_results()
        self._close_connection()
        self.state = self.CLOSED

    # internal
    def _send_failed(self, data):
        if not self.finished:
            self.finished = True
            self.consumer.http_failed(data)
    
    def _send_results(self):
        if not self.finished:
            self.finished = True
            if self.header and self.header.getheader(
                'content-encoding') == 'gzip':
                self._data = gzip.GzipFile(
                    fileobj = StringIO(self._data)).read()
            self.consumer.http_results(self.status, self.header, self._data)

    def _close_connection(self):
        if hasattr(self, "connection"):
            try:
                self.connection.close()
            except:
                # silently ignore errors: if it didn't succeed, it wasn't
                # open in the first place. we don't care.
                pass
            del self.connection

    def __str__(self):
        host = "Unknown"
        path = "Unknown"
        if getattr(self, "request", None) is not None:
            host = self.request.host
            path = self.request.path
        return '<%s for %s %s>' % (repr(self), host, path)




""" MessageManager.py

Manages status messages
"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """
Straw is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Straw is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """

import Event

class MessageManager(Event.SignalEmitter):
    def __init__(self):
        Event.SignalEmitter.__init__(self)
        self.initialize_slots(Event.StatusDisplaySignal)
        self.__messages = []

    def post_message(self, message):
        self.__messages.append(message)
        self.emit_signal(Event.StatusDisplaySignal(self))

    def read_message(self):
        return self.__messages.pop(0)

    def number_of_messages(self):
        return len(self.__messages)

mm_instance = None

def get_instance():
    global mm_instance
    if mm_instance is None:
        mm_instance = MessageManager()
    return mm_instance




#! /usr/bin/python

import unittest
import sys
import random

sys.path.insert(0, '../src/lib')
import QueueDict

class QueueDictTestCase(unittest.TestCase):
    def setUp(self):
        self.N_ITEMS = 100
        self._qd = QueueDict.FeedQD()
        self._order = False
        self._qd.sort_order = self._order

        for value in xrange(self.N_ITEMS):
            for key in random.sample(xrange(self.N_ITEMS), self.N_ITEMS):
                self._qd[key] = "Item#%d" % value

    def getKeys(self):
        return self._qd.keys()

    def pickRandomKey(self, keys):
        return random.choice(keys)

    def getIndex(self, key):
        return self._qd.index(key)

    def testIndex(self):
        keys = self.getKeys()
        key = self.pickRandomKey(keys)
        self.assertEqual(keys.index(key), self._qd.index(key))

    def testKeys(self):
        self.failUnless(self.getKeys() > 0)

    def testLength(self):
        self.assertEqual (self.N_ITEMS, len(self._qd))

    def testSortOrder(self):
        self.assertEqual (self._order, self._qd.sort_order)
        self._qd.sort_order = not self._order
        self.assertNotEqual (self._order, self._qd.sort_order)

    def testFirstItem(self):
        def tfi():
            items = self._qd.items()
            front = self._qd.get_first_item()
            self.failUnless(front is not None)
            self.assertEqual(items[0][1], front)
        # normal order
        tfi()
        # reversed order
        self._qd.sort_order = not self._order
        tfi()

    def testLastItem(self):
        def lit():
            items = self._qd.items()
            back = self._qd.get_last_item()
            self.failUnless(back is not None)
            self.assertEqual(items[-1][1], back)
        # normal order
        lit()
        # reversed order
        self._qd.sort_order = not self._order
        lit()

    def testNextItem(self):
        keys = self.getKeys()
        random_key = self.pickRandomKey(keys)
        next_keyidx = self.getIndex(random_key) + 1
        next_item = self._qd.get_next_item(random_key)

        self.failUnless(next_keyidx is not None)
        self.failUnless(next_item is not None)
        self.assertEqual(self._qd.get(keys[next_keyidx]), next_item)

    def testPreviousItem(self):
        keys = self.getKeys()
        random_key = self.pickRandomKey(keys)
        prev_keyidx = self.getIndex(random_key) - 1
        prev_item = self._qd.get_previous_item(random_key)

        self.failUnless(prev_keyidx is not None)
        self.failUnless(prev_item is not None)
        self.assertEqual(self._qd.get(keys[prev_keyidx]), prev_item)


    def testNonExistentKey(self):
        # Value of a non-existent key should be None
        key = chr(int(random.random() * self.N_ITEMS))
        item = self._qd.get(key)
        self.failUnless (item is None)

def suite():
    suite = unittest.makeSuite(TestQueueDict, 'test')
    return suite

if __name__ == '__main__':
    unittest.main()




""" dialogs.py

Module for displaying dialogs related to errors, warnings, notifications,
etc...
"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """
Straw is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Straw is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """

import pygtk
pygtk.require('2.0')
import gobject
import gtk
import gnome
import error
import FeedCategoryList
import FeedList
import OPMLExport
import OPMLImport
import utils
import constants
import gnome.ui
import os, os.path
import gettext


def _setup_filechooser_dialog(title, action, extra_widget_title):
    """
    Setup the file chooser dialog. This includes an extra widget (a combobox)
    to include the categories to import or export
    """
    dialog = gtk.FileChooserDialog(title, action=action,
                                   buttons=(gtk.STOCK_CANCEL,
                                            gtk.RESPONSE_CANCEL,
                                            gtk.STOCK_OK, gtk.RESPONSE_OK))
    category_list = FeedCategoryList.get_instance()
    model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
    combobox = gtk.ComboBox(model)
    celltitle = gtk.CellRendererText()
    combobox.pack_start(celltitle,False)
    combobox.add_attribute(celltitle, 'text', 0)

    # add 'All' Category
    it = model.append()
    model.set(it, 0, category_list.all_category.title,
              1, category_list.all_category)

    # add user categories
    for category in category_list.user_categories:
        it = model.append()
        model.set(it, 0, category.title, 1, category)

    # ..
    combobox.set_active(0)
    label = gtk.Label(extra_widget_title)
    label.set_alignment(1.0,0.5)
    hbox = gtk.HBox(spacing=6)
    hbox.pack_start(label,True,True,0)
    hbox.pack_end(combobox,False,False,0)
    hbox.show_all()

    dialog.set_extra_widget(hbox)
    return (dialog, combobox)


def export_subscriptions(parent):
    (dialog,combobox) = _setup_filechooser_dialog(_("Export Subscriptions"),
                                                  gtk.FILE_CHOOSER_ACTION_SAVE,
                                                  _("Select category to export:"))
    def selection_changed(widget, dialog):
        model = widget.get_model()
        category = model[widget.get_active()][1]
        dialog.set_current_name("Straw-%s.xml" % category.title)

    combobox.connect('changed',
                     selection_changed,
                     dialog)
    selection_changed(combobox, dialog)
    dialog.set_transient_for(parent)
    response = dialog.run()
    if response == gtk.RESPONSE_OK:
        filename = dialog.get_filename()
        model = combobox.get_model()
        cat = model[combobox.get_active()][1]
        OPMLExport.export(cat.title, cat.feeds, filename)
    dialog.destroy()

def import_subscriptions(parent):
    (dialog,combobox) = _setup_filechooser_dialog(_("Import Subscriptions"),
                                                  gtk.FILE_CHOOSER_ACTION_OPEN,
                                                  _("Add new subscriptions in:"))
    ffilter = gtk.FileFilter()
    ffilter.set_name(_("OPML Files Only"))
    ffilter.add_pattern("*.xml")
    ffilter.add_pattern("*.opml")
    dialog.add_filter(ffilter)
    dialog.set_transient_for(parent)
    response = dialog.run()
    if response == gtk.RESPONSE_OK:
        filename = dialog.get_filename()
        model = combobox.get_model()
        cat = model[combobox.get_active()][1]
        dialog.hide()
        OPMLImport.import_opml(filename, cat)
    dialog.destroy()

def report_error(primary, secondary, parent=None):
    dialog = gtk.MessageDialog(parent,
                               type=gtk.MESSAGE_ERROR,
                               buttons=gtk.BUTTONS_OK,
                               message_format=primary)
    dialog.format_secondary_text(secondary)
    response = dialog.run()
    dialog.destroy()
    return response

def report_offline_status(parent=None):
    dialog = gtk.MessageDialog(parent,
                               type=gtk.MESSAGE_WARNING,
                               buttons=gtk.BUTTONS_OK_CANCEL,
                               message_format="You Are Offline!")
    dialog.format_secondary_markup(_("You are currently reading offline. Would you like to go online now?"))
    response = dialog.run()
    dialog.destroy()
    return response

def confirm_delete(primary, secondary,parent=None):
    dialog = gtk.MessageDialog(parent,
                               type=gtk.MESSAGE_QUESTION,
                               buttons=gtk.BUTTONS_OK_CANCEL,
                               message_format=primary)
    dialog.format_secondary_text(secondary)
    response = dialog.run()
    dialog.hide()
    return response

def credits():
    iconfile = os.path.join(utils.find_image_dir(),"straw.png")
    logo = gtk.gdk.pixbuf_new_from_file(iconfile)
    description = _("A feed aggregator for the GNOME Desktop")
    straw_copyright = u"""
    Copyright \xa9 2002-2004 Juri Pakaste
    Copyright \xa9 2005-2007 Straw Contributors"""
    people = [
        u"Iain McCoy <iain@mccoy.id.au>",
        u"Jan Alonzo <jmalonzo@unpluggable.com>",
        u"Juri Pakaste <juri@iki.fi>",
        u"Leandro Lameiro <lameiro@gmail.com>",
        u"Lucas Nussbaum <lucas@lucas-nussbaum.net>",
        u"Mark Pilgrim (feedparser and feedfinder)",
        u"Olivier Crete <tester@tester.ca>",
        u"Ryan P. Skadberg <skadz@stigmata.org>",
        u"Scott Douglas-Watson <sdouglaswatson@yahoo.co.uk>",
        u"Terje R\xf8sten (distutils)",
        u"Tuukka Hastrup <tuukka@iki.fi>"]

    artists = [
        u"Jakub 'jimmac' Steiner"
        ]

    translators = [
        u"GNOME i18n Project and Translators<http://developer.gnome.org/projects/gtp/>",
        u"Juri Pakaste <juri@iki.fi>",
        u"Martin Steldinger <tribble@hanfplantage.de>",
        u"David Rousseau <boiteaflood@wanadoo.fr>",
        u"Sergei Vavinov <svv@cmc.msu.ru>",
        u"Terje R\xf8sten <terjeros@phys.ntnu.no>",
        u"Francisco J. Fernandez <franciscojavier.fernandez.serrador@hispalinux.es>",
        u"Elros Cyriatan (Dutch Translation)"
        ]

    translator_credits = 'translator_credits'
    about = gtk.AboutDialog()
    about.set_name(constants.APPNAME)
    about.set_version(constants.VERSION)
    about.set_copyright(straw_copyright)
    about.set_comments(description)
    about.set_authors(people)
    artists.append(u'Juri Pakaste')
    about.set_artists(artists)
    about.set_logo(logo)
    about.set_translator_credits(translator_credits)
    about.set_license(__license__)
    gtk.about_dialog_set_url_hook(lambda about, url: utils.url_show(url))
    about.set_website(constants.STRAW_URL)
    about.set_website_label(constants.STRAW_URL)
    about.connect('response', lambda widget, response: widget.destroy())
    return about



""" error.py

Module for logging errors
"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """ GNU General Public License

This program is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """

import sys
import traceback
import pprint
from StringIO import StringIO

stream = sys.stderr

def log(*args):
    caller = traceback.extract_stack()[-2]
    _log(caller, *args)

def _log(caller = None, *args):
    if caller is None:
        caller = traceback.extract_stack()[-2]
    try:
        cf = caller[0][caller[0].rindex("/")+1:]
    except:
        cf = caller[0]
    stream.write("%s:%d:%s: " % (cf, caller[1], caller[2]))
    for a in args:
        if type(a) is not type(''):
            a = repr(a)
        stream.write(a)
    stream.write("\n")

def logtb(*args):
    _log(traceback.extract_stack()[-2], *args)
    l = traceback.format_list(traceback.extract_stack()[:-1])
    for e in l:
        stream.write(e)

def logparam(locals, *vars):
    logargs = []
    for v in vars:
        logargs.append(v + ": " + str(locals[v]))
    _log(traceback.extract_stack()[-2], ", ".join(logargs))
    
def logpparam(locals, *vars):
    logargs = []
    for v in vars:
        sio = StringIO()
        pprint.pprint(locals[v], sio)
        logargs.append(v + ": " + sio.getvalue())
        sio.close()
    _log(traceback.extract_stack()[-2], "".join(logargs))


depth = 0
def incr_depth():
    global depth
    depth += 1
def decr_depth():
    global depth
    depth -= 1
def get_indent(): return " " * depth

def debug_around(f):
    caller = traceback.extract_stack()[-2]
    
    def f2(*args, **kwargs):
        def maybe_str(o):
            if isinstance(o, str):
                return repr(o)
            return str(o)
        indent = get_indent()
        _log(caller, "%sEntering " % indent, f.__name__)
        argstr = ", ".join([maybe_str(a) for a in args])
        if len(kwargs) > 0:
            argstr += ", " + ", ".join(
                ["%s: %s" % (maybe_str(k), maybe_str(v))
                 for k, v in kwargs.items()])
        _log(caller, "%sArgs: (" % indent, argstr, ")")
        incr_depth()
        r = None
        try:
            r = f(*args, **kwargs)
        finally:
            decr_depth()
            _log(caller, "%sReturning %s from %s" % (
                indent, str(r), f.__name__))
            return r


    return f2

# Originally by Bryn Keller
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/52215
def log_exc(message):
    """
    Print the usual traceback information, followed by a listing of all the
    local variables in each frame.
    """
    _log(traceback.extract_stack()[-2], message)
    tb = sys.exc_info()[2]
    while 1:
        if not tb.tb_next:
            break
        tb = tb.tb_next
    stack = []
    f = tb.tb_frame
    while f:
        stack.append(f)
        f = f.f_back
    stack.reverse()
    traceback.print_exc()
    print >>stream, "Locals by frame, innermost last"
    for frame in stack:
        print >>stream
        print >>stream, "Frame %s in %s at line %s" % (
            frame.f_code.co_name, frame.f_code.co_filename,
            frame.f_lineno)
        for key, value in frame.f_locals.items():
            print >>stream, "\t%20s = " % key,
            #We have to be careful not to cause a new error in our error
            #printer! Calling str() on an unknown object could cause an
            #error we don't want.
            try:                   
                print >>stream, value
            except:
                print >>stream, "<ERROR WHILE PRINTING VALUE>"



""" ValueMonitor.py

"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """
Straw is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Straw is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """


import pygtk
pygtk.require('2.0')
import gobject
import error

class ValueMonitor(object):
    def __init__(self, current, timeout, notify, data):
        self._previous_value = current
        self._value = None
        self._timeout = timeout
        self._notify = notify
        self._data = data
        self._connected = True
        self._value_changed = False

    def get_value(self):
        return self._value

    def set_value(self, value):
        if self._connected:
            if value != self._value:
                self._value_changed = True
                gobject.timeout_add(self._timeout, self._check_value)
            self._value = value

    value = property(get_value, set_value)

    def connect(self):
        self._connected = True

    def disconnect(self):
        self.flush()
        self._connected = False

    def flush(self):
        self._check_value()

    def _check_value(self):
        if (self._connected and self._value_changed and self.value != self._previous_value):
            self._notify(self.value, self._data)
            self._previous_value = self.value

def create_for_gtkentry(widget, timeout, notify):
    vm = ValueMonitor(widget.get_text(), timeout, notify, widget)
    def changed(object, *data):
        vm.value = object.get_text()
    widget.connect("changed", changed)
    return vm

def create_for_gtkspin(widget, timeout, notify):
    vm = ValueMonitor(widget.get_value_as_int, timeout, notify, widget)
    def changed(object, *data):
        vm.value = object.get_value_as_int()
    widget.connect("value-changed", changed)
    return vm



""" ParsedSummary.py

"""

__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """
Straw is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Straw is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """

class ParsedSummary(object):
    __slots__ = ('title', 'link', 'description', 'last_build_date', 'contributors',
                 'managing_editor', 'web_master', 'language', 'items', 'copyright', 'creator')

    def __init__(self):
        self.items = []
        self.last_build_date = -1
        self.title = ""
        self.description = ""
        self.copyright = ""
        self.link = ""
        self.language = ""
        self.creator = ""
        self.contributors = list()

    def addItem(self, item):
        self.items.append(item)




""" ItemStore.py

Data store abstraction module.
"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """
Straw is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Straw is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """

import cPickle as pickle
import os, sys
import time
from error import *
import tempfile
import traceback

try:
    from bsddb.db import *
    import bsddb
except ImportError:
    from bsddb3.db import *
    import bsddb3 as bsddb

import Event
import SummaryItem
import FeedList
import ImageCache
from MainloopManager import MainloopManager

DATABASE_FILE_NAME = "itemstore.db"

class ConvertException(Exception):
    def __init__(self, version1, version2, reason):
        self.version1 = version1
        self.version2 = version2
        self.reason = reason

class MyDB:
    CURRENT_VERSION = 3

    def __init__(self, filename, dbhome, create=0, truncate=0, mode=0600,
                 recover=0, dbflags=0):
        self._db = None
        self._env = None
        recoverenv = DB_CREATE | DB_RECOVER
        # DB_INIT_TXN automatically enables logging
        flagsforenv = DB_INIT_TXN | DB_INIT_MPOOL | DB_INIT_LOCK | DB_PRIVATE

        self._env = DBEnv()
        self._env.set_data_dir(dbhome)
        self._env.set_lk_detect(DB_LOCK_DEFAULT)  # enable auto deadlock avoidance
        self._env.set_lg_max(2**20)
        self._env.set_lk_max_locks   (10000)
        self._env.set_lk_max_objects (10000)

        try:
            self._env.open(dbhome, recoverenv | flagsforenv, mode)
        except bsddb._db.DBRunRecoveryError, err:
            self._env.remove(dbhome)
            self._env.close()
            log("%s" % err[1])
            sys.exit("Recovery Error: See README for details on how to recover data. ")

        flags = 0
        if truncate:
            flags |= DB_TRUNCATE

        try:
            flags |= DB_AUTO_COMMIT
        except NameError:
            pass

        try:
            self._db = DB(self._env)
            self._db.open(filename, DB_BTREE, flags, mode)
        except bsddb._db.DBNoSuchFileError:
            if create:
                self._db = DB(self._env)
                self._db.open(filename, DB_BTREE, flags | DB_CREATE, mode)
                self.set_db_version(self.CURRENT_VERSION)
            else:
                raise
        try:
            self.convert_old_versions()
        except Exception, ex:
            try:
                filename = tempfile.mktemp(prefix="straw-")
                fh = open(filename, "w")
                traceback.print_exc(None, fh)
                raise ConvertException(self.get_db_version(),
                                       MyDB.CURRENT_VERSION, "%s" % filename)
            finally:
                fh.close()

    def close(self):
        if self._db is not None:
            self._db.close()
            self._db = None
        if self._env is not None:
            self._env.close()
            self._env = None

    def checkpoint(self):
        # set flags to 0 or DB_FORCE, else will raise EINVAL (InvalidArgError)
        cpflags = 0
        self._env.txn_checkpoint(cpflags | DB_FORCE )
        deletees = self._env.log_archive(DB_ARCH_ABS)
        for d in deletees:
            os.remove(d)

    def begin_transaction(self):
        return self._env.txn_begin()

    def get_item_ids(self, iid, txn):
        key = "fids:%d" % iid
        dids = self._db.get(key, txn=txn)
        ids = []
        if dids:
            ids = pickle.loads(dids)
        return ids

    def save_feed_item_ids(self, feed, ids, txn=None):
        rowid = "fids:%d" % feed.id
        commit = 0
        if not txn:
            txn = self.begin_transaction()
            commit = 1
        try:
            try:
                self._db.delete(rowid, txn=txn)
            except DBNotFoundError:
                pass
            self._db.put(rowid, pickle.dumps(ids), txn=txn)
        except Exception, ex:
            if commit:
                txn.abort()
            logtb(str(ex))
        else:
            if commit:
                txn.commit()

    def get_item(self, feed_id, item_id, txn=None):
        item = self._db.get("%d:%d" % (feed_id, item_id), txn=txn)
        return unstringify_item(item)

    def add_items(self, feed, items):
        txn = self.begin_transaction()
        try:
            feed_item_ids = self.get_item_ids(feed.id, txn=txn)
            for item in items:
                self._db.put("%d:%d" % (item.feed.id, item.id), stringify_item(item), txn=txn)
                # TODO: it might be a good idea to check here that we don't add
                # duplicate items. It doesn't happen normally, but there can be
                # bugs that trigger that. Throwing an exception would be the
                # the right thing: it wouldn't hide the breakage.
                feed_item_ids.append(item.id)
            self.save_feed_item_ids(feed, feed_item_ids, txn)
        except Exception, ex:
            txn.abort()
            logtb(str(ex))
        else:
            txn.commit()

    def delete_items(self, feed, items):
        """ Deletes a list of items.

        Useful for cutting old items based on number of items stored.
        """
        txn = self.begin_transaction()
        try:
            feed_item_ids = self.get_item_ids(feed.id, txn=txn)
            # because of bugs, we sometime get here duplicate ids. instead of dying,
            # warn the user but continue
            item_ids = []
            for item in items:
                item.clean_up()
                if item.id in item_ids:
                    log("WARNING: skipping duplicate ids in delete items request %s and %s" % (item.title, item.id))
                    # filter out any duplicates
                    feed_item_ids = filter(lambda x: x != item.id, feed_item_ids)
                    continue
                item_ids.append(item.id)
                #log("deleting item %d:%d" % (feed.id, item.id))
                if item.id in feed_item_ids:
                    feed_item_ids.remove(item.id)
                    self._db.delete("%d:%d" % (feed.id, item.id), txn=txn)
            self.save_feed_item_ids(feed, feed_item_ids, txn)
        except Exception, ex:
            txn.abort()
            log_exc("error while deleting items")
        else:
            txn.commit()

    def modify_items(self, items):
        txn = self.begin_transaction()
        try:
            for item in items:
                self._db.put("%d:%d" % (item.feed.id, item.id),
                             stringify_item(item), txn=txn)
        except Exception, ex:
            txn.abort()
            logtb(str(ex))
        else:
            txn.commit()

    def get_feed_items(self, feed):
        txn = self.begin_transaction()
        items = []
        try:
            ids = self.get_item_ids(feed.id, txn=txn)
            for id in ids:
                item = self.get_item(feed.id, id, txn=txn)
                if item is not None:
                    items.append(item)
        except Exception, ex:
            txn.abort()
            log(str(ex))
        else:
            txn.commit()
            return items

    def get_number_of_unread(self, fid, cutoff):
        # Used by config conversion
        # NOTE: this is the number of unread items in 'number of items stored'
        # preference. Since straw stores the most recent items down the list,
        # we only count the unread items from the most recent N items,
        # where N = cutoff.
        txn = self.begin_transaction()
        num_unread = 0
        try:
            ids = self.get_item_ids(fid, txn=txn)
            for id in ids[len(ids)-cutoff:]:
                item = self.get_item(fid, id, txn=txn)
                if item is not None and item.seen == 0:
                    num_unread += 1
                else: continue
        except Exception, ex:
            txn.abort()
            logtb(str(ex))
        else:
            txn.commit()
            return num_unread

    def get_image_urls(self, txn=None):
        dkeys = self._db.get("images", txn=txn)
        keys = []
        if dkeys is not None:
            keys = pickle.loads(dkeys)
        return keys

    def save_image_urls(self, urls, txn=None):
        self._db.put("images", pickle.dumps(urls), txn=txn)

    def get_image_counts(self, txn=None):
        images = self.get_image_urls(txn)
        counts = []
        for image in images:
            key = ("imagecount:" + image).encode('utf-8')
            value = self._db.get(str(key))
            try:
                counts.append((image, int(value)))
            except:
                log("exception for ", key, ", type of value ", value, ": ", type(value))
        return counts

    def update_image_count(self, url, count):
        #logparam(locals(), "url", "count")
        key = ("imagecount:" + url).encode('utf-8')
        txn = self.begin_transaction()
        try:
            if count < 1:
                self._db.delete(key, txn=txn)
            else:
                self._db.put(key, str(count), txn=txn)
        except:
            txn.abort()
            raise
        else:
            txn.commit()

    def update_image(self, url, image):
        key = "image:%s" % str(url)
        txn = self.begin_transaction()
        try:
            image_urls = self.get_image_urls(txn)
            if image is not None:
                self._db.put(key.encode('utf-8'), image, txn=txn)
                if url not in image_urls:
                    image_urls.append(url)
                    self.save_image_urls(image_urls, txn)
            else:
                if url in image_urls:
                    try:
                        self._db.delete(key, txn=txn)
                    except DBNotFoundError:
                        log("Key not found", key)
                    image_urls.remove(url)
                    self.save_image_urls(image_urls, txn=txn)
        except:
            txn.abort()
            raise
        else:
            txn.commit()

    def get_image_data(self, url, txn=None):
        return self._db.get(
            "image:%s" % url.encode('utf-8'), default = None, txn=txn)

    def _image_print(self, key, data):
        if key[:6] == "image:":
            print key

    def _data_print(self, key, data):
        data = pickle.loads(data)
        pprint ({key: data})

    def _db_print(self, helper):
        """Print the database to stdout for debugging"""
        print "******** Printing raw database for debugging ********"
        print "database version: %s" % self.get_db_version()
        cur = self._db.cursor()
        try:
            key, data = cur.first()
            while 1 :
                helper(key, data)
                next = cur.next()
                if next:
                    key, data = next
        finally:
            cur.close()

    def get_db_version(self, txn=None):
        version = self._db.get("straw_db_version", default = "1", txn=txn)
        return int(version)

    def set_db_version(self, version, txn=None):
        try:
            if txn is None:
                txn = self.begin_transaction()
                self._db.put("straw_db_version", str(version), txn=txn)
        except:
            txn.abort()
            raise
        else:
            txn.commit()

    def convert_old_versions(self):
        version = self.get_db_version()
        while version < self.CURRENT_VERSION:
            next = version + 1
            mname = "convert_%d_%d" % (version, next)
            try:
                method = getattr(self, mname)
            except AttributeError:
                raise ConvertException(version, next, "No conversion function specified")
            method()
            self.set_db_version(next)
            version = next

    def convert_1_2(self):
        def is_item(key):
            parts = key.split(':')
            if len(parts) != 2:
                return False
            return parts[0].isdigit() and parts[1].isdigit()

        def round_second(ttuple):
            l = list(ttuple)
            l[5] = int(round(l[5]))
            return tuple(l)

        try:
            import mx.DateTime as mxd
        except ImportError:
            raise ConvertException(1, 2, _("Couldn't import mx.DateTime"))
        txn = self.begin_transaction()
        try:
            cur = self._db.cursor(txn=txn)
            try:
                next = cur.first()
                key = None
                if next:
                    key, data = cur.first()
                while key is not None:
                    if is_item(key):
                        dict = pickle.loads(data)
                        if isinstance(dict['pub_date'], mxd.DateTimeType):
                            p = dict['pub_date']
                            t = time.gmtime(time.mktime(round_second(p.tuple())))
                            dict['pub_date'] = t
                            data = pickle.dumps(dict)
                            cur.put(key, data, DB_CURRENT)
                    next = cur.next()
                    if next:
                        key, data = next
                    else:
                        break
            finally:
                cur.close()
        except Exception, ex:
            txn.abort()
            raise
        else:
            txn.commit()

    def convert_2_3(self):
        def is_item(key):
            parts = key.split(':')
            if len(parts) != 2:
                return False
            return parts[0].isdigit() and parts[1].isdigit()

        imagelistcursor = None
        images = {}
        txn = self.begin_transaction()
        try:
            cur = self._db.cursor(txn=txn)
            try:
                next = cur.first()
                key = None
                if next:
                    key, data = cur.first()
                while key is not None:
                    if is_item(key):
                        dic = pickle.loads(data)
                        for image in dic['images']:
                            images[image] = images.get(image, 0) + 1
                    elif key == "images":
                        imagelistcursor = cur.dup(DB_POSITION)
                    next = cur.next()
                    if next:
                        key, data = next
                    else:
                        break
                for image, count in images.items():
                    key = ("imagecount:" + image).encode('utf-8')
                    cur.put(key, str(count), DB_KEYFIRST)
                imagelistcursor.put("images", pickle.dumps(images.keys()), DB_CURRENT)
            finally:
                cur.close()
                if imagelistcursor != None:
                    imagelistcursor.close()
        except Exception, ex:
            txn.abort()
            raise
        else:
            txn.commit()

class ModifyItemAction:
    def __init__(self, item):
        self._item = item

    def doit(self, db):
        db.modify_items([self._item])

class ModifyItemsAction:
    def __init__(self, items):
        self._items = items

    def doit(self, db):
        db.modify_items(self._items)

class ItemsAddedAction:
    def __init__(self, feed, items):
        self._feed = feed
        self._items = items

    def doit(self, db):
        db.add_items(self._feed, self._items)

class DeleteItemAction:
    def __init__(self, feed, items):
        self._feed = feed
        self._items = items

    def doit(self, db):
        db.delete_items(self._feed, self._items)

class ImageUpdateAction:
    def __init__(self, url, image):
        self._url = url
        self._image = image

    def doit(self, db):
        db.update_image(self._url, self._image)

class ImageCountChangedAction:
    def __init__(self, url, count):
        self._url = url
        self._count = count

    def doit(self, db):
        db.update_image_count(self._url, self._count)

class ItemStore:
    def __init__(self, dbhome):
        feedlist = FeedList.get_instance()
        self._db = MyDB(DATABASE_FILE_NAME, dbhome, create = 1)
        self.connect_signals()
        feedlist.signal_connect(Event.FeedCreatedSignal,
                                self._feed_created_cb)
        feedlist.signal_connect(Event.FeedDeletedSignal,
                                self._feed_deleted_cb)
        ImageCache.cache.signal_connect(Event.ImageUpdatedSignal,
                                        self.image_updated)
        self._stop = False
        self._action_queue = []

    def _feed_created_cb(self, signal):
        self._connect_feed_signals(signal.feed)

    def _feed_deleted_cb(self, signal):
        self._disconnect_feed_signals(signal.feed)

    def connect_signals(self):
        flist = FeedList.get_instance().flatten_list()
        for f in flist:
            self._connect_feed_signals(f)

    def _connect_feed_signals(self, feed):
        feed.signal_connect(Event.NewItemsSignal, self.items_added)
        feed.signal_connect(Event.ItemReadSignal, self.item_modified)
        feed.signal_connect(Event.ItemStickySignal, self.item_modified)
        feed.signal_connect(Event.AllItemsReadSignal, self.all_items_read)
        feed.signal_connect(Event.ItemDeletedSignal, self.item_deleted)

    def _disconnect_feed_signals(self, feed):
        feed.signal_disconnect(Event.NewItemsSignal, self.items_added)
        feed.signal_disconnect(Event.ItemReadSignal, self.item_modified)
        feed.signal_disconnect(Event.ItemStickySignal, self.item_modified)
        feed.signal_disconnect(Event.AllItemsReadSignal, self.all_items_read)
        feed.signal_disconnect(Event.ItemDeletedSignal, self.item_deleted)


    def modify_item(self, item):
        self._action_queue.append(ModifyItemAction(item))
        return

    def image_updated(self, signal):
        self._action_queue.append(
            ImageUpdateAction(signal.url, signal.data))

    def read_image(self, url):
        return self._db.get_image_data(url)

    def item_deleted(self, signal):
        self._action_queue.append(DeleteItemAction(signal.sender, signal.item))

    def item_modified(self, signal):
        self.modify_item(signal.item)

    def all_items_read(self, signal):
        self._action_queue.append(ModifyItemsAction(
            [item for index, item in signal.changed]))

    def items_added(self, signal):
        self._action_queue.append(
            ItemsAddedAction(signal.sender, signal.items))

    def read_feed_items(self, feed):
        return self._db.get_feed_items(feed)

    def get_number_of_unread(self, feed_id, cutoff):
        return self._db.get_number_of_unread(feed_id, cutoff)

    def get_image_counts(self):
        return self._db.get_image_counts()

    def set_image_count(self, image, count):
        self._action_queue.append(
            ImageCountChangedAction(image, count))

    def start(self):
        mlmgr = MainloopManager.get_instance()
        mlmgr.set_repeating_timer(5000, self._run)

    def stop(self):
        mlmgr = MainloopManager.get_instance()
        mlmgr.end_repeating_timer(self._run)
        self._db.checkpoint()
        self._db.close()
        self._stop = True

    def _run(self):
        self._db.checkpoint()
        freq = 5
        timer = freq
        cpfreq = 60
        cptimer = cpfreq
        prevtime = time.time()
        if not self._stop:
            tmptime = time.time()
            timer += tmptime - prevtime
            cptimer += tmptime - prevtime
            prevtime = tmptime
            if timer > freq:
                try:
                    while len(self._action_queue):
                        action = self._action_queue.pop(0)
                        if action is None:
                            break
                        action.doit(self._db)
                except IndexError, e:
                    pass
                timer = 0
            if cptimer > cpfreq:
                self._db.checkpoint()
                cptimer = 0

itemstore_instance = None
def get_instance(straw_dir=None):
    global itemstore_instance
    if itemstore_instance is None:
        if straw_dir is None:
            straw_dir = os.path.join(os.getenv('HOME'), ".straw")
        itemstore_instance = ItemStore(straw_dir)
    return itemstore_instance

def stringify_item(item):
    itemdict = {
        'title': item.title,
        'link': item.link,
        'description': item.description,
        'guid': item.guid,
        'guidislink': item.guidislink,
        'pub_date': item.pub_date,
        'source': item.source,
        'images': item.image_keys(),
        'seen': item.seen,
        'id': item.id,
        'fm_license': item.fm_license,
        'fm_changes': item.fm_changes,
        'creator': item.creator,
        'contributors': item.contributors,
        'license_urls': item.license_urls,
        'publication_name': item.publication_name,
        'publication_volume': item.publication_volume,
        'publication_number': item.publication_number,
        'publication_section': item.publication_section,
        'publication_starting_page': item.publication_starting_page,
        'sticky': item._sticky,
        'enclosures': item.enclosures}
    return pickle.dumps(itemdict)

def unstringify_item(itemstring):
    if not itemstring:
        return None

    idict = _unpickle(itemstring)
    if not idict:
        return None

    item = SummaryItem.SummaryItem()
    item.title = idict['title']
    item.link = idict['link']
    item.description = idict['description']
    item.guid = idict['guid']
    item.pub_date = idict['pub_date']
    item.source = idict['source']
    for i in idict['images']:
        item.restore_image(i)
    item.seen = idict['seen']
    item.id = idict['id']
    item.guidislink = idict.get('guidislink', True)
    item.fm_license = idict.get('fm_license', None)
    item.fm_changes = idict.get('fm_changes', None)
    item.creator = idict.get('creator', None)
    item.contributors = idict.get('contributors', None)
    item.license_urls = idict.get('license_urls', None)
    item._sticky = idict.get('sticky', 0)
    item.enclosures = idict.get('enclosures', None)
    item.publication_name = idict.get('publication_name', None)
    item.publication_volume = idict.get('publication_volume', None)
    item.publication_number = idict.get('publication_number', None)
    item.publication_section = idict.get('publication_section', None)
    item.publication_starting_page = idict.get('publication_starting_page', None)
    return item

def _unpickle(istring):
    itemdict = None
    try:
        itemdict = pickle.loads(istring)
    except ValueError, ve:
        log("ItemStore.unstringify_item: pickle.loads raised ValueError, argument was %s" % repr(itemstring))
    except Exception, ex:
        logtb(str(ex))
    return itemdict

if __name__ == '__main__':
    from pprint import pprint
    db = MyDB("itemstore.db", "%s/.straw" % os.getenv('HOME'), create = 1)
    db._db_print(db._data_print)



""" FeedPropertiesDialog.py

Provides a module for handling the properties of a feed
"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """
Straw is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Straw is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """


from error import log, logparam, logtb
from xml.sax import saxutils
import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import gobject
import gnome
import time
import utils
import Feed
import Config
import FeedCategoryList
import SummaryParser
import gettext

class FeedPropertiesDialog(object):
    COLUMN_NAME = 0
    COLUMN_MEMBER = 1
    COLUMN_OBJECT = 2

    def __init__(self, parent, feed, deleter):
        gladef = utils.find_glade_file()
        xml = gtk.glade.XML(gladef, 'feed_properties_dialog', gettext.textdomain())
        self._window = xml.get_widget('feed_properties_dialog')
        self._window.set_transient_for(parent)
        self._title = xml.get_widget('properties_title_entry')
        self._location = xml.get_widget('properties_location_entry')
        self._username = xml.get_widget('properties_username_entry')
        self._password = xml.get_widget('properties_password_entry')
        self._username_title = xml.get_widget('properties_username_label')
        self._password_title = xml.get_widget('properties_password_label')
        self._restore_button = xml.get_widget('properties_reset_button')
        self._refresh_spin = xml.get_widget('properties_refresh_spin')
        self._articles_spin = xml.get_widget('properties_articles_spin')
        self._refresh_default_check = xml.get_widget(
            'properties_keep_refresh_default')
        self._articles_default_check = xml.get_widget(
            'properties_keep_articles_default')
        self._next_refresh_label = xml.get_widget(
            'properties_next_refresh_label')
        self._previous_refresh_label = xml.get_widget(
            'properties_previous_refresh_label')
        self._categories_treeview = xml.get_widget('feed_categories_treeview')
        self._feed_info_href_container = xml.get_widget(
            'feed_info_href_container')
        self._feed_info_description = xml.get_widget('feed_info_description')
        self._feed_info_copyright = xml.get_widget('feed_info_copyright')

        self._feed_info_href = gnome.ui.HRef(url='', text='')
        self._feed_info_href_container.pack_start(
            self._feed_info_href, expand=False, fill=False)

        self._title_default = ''
        self._url_default = ''
        self._un_default = ''
        self._pw_default = ''

        self._size_group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
        self._size_group.add_widget(xml.get_widget('properties_title_label'))
        self._size_group.add_widget(xml.get_widget('properties_location_label'))
        self._size_group.add_widget(xml.get_widget('properties_username_label'))
        self._size_group.add_widget(xml.get_widget('properties_password_label'))

        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_BOOLEAN,
                              gobject.TYPE_PYOBJECT)
        self._categories_treeview.set_model(model)
        self._categories_treeview.set_rules_hint(True)

        r = gtk.CellRendererToggle()
        r.connect('toggled', self.member_toggled)
        column = gtk.TreeViewColumn(_('Member'), r, active=self.COLUMN_MEMBER)
        self._categories_treeview.append_column(column)

        r = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_('Name'), r, text=self.COLUMN_NAME)
        self._categories_treeview.append_column(column)

        nameFuncMap = {}
        for key in dir(self.__class__):
            if key[:3] == 'on_':
                nameFuncMap[key] = getattr(self, key)
        xml.signal_autoconnect(nameFuncMap)

        self._initializing_window = False
        self._feed = feed
        self.deleter = deleter

    def get_state(self):
        return self._window.window.get_state()

    def show(self):
        self._initializing_window = True
        self.display_properties()
        self._window.present()
        self._initializing_window = False

    def hide(self, *args):
        self.deleter()
        self._window.hide()
        self._window.destroy()

    def on_feed_properties_dialog_delete_event(self, *args):
        self.hide()
        return True

    def on_feed_properties_close_button_clicked(self, *args):
        self.hide()
        return

    def member_toggled(self, cell, path):
        model = self._categories_treeview.get_model()
        category = model[path][self.COLUMN_OBJECT]
        if self._feed in category.feeds:
            category.remove_feed(self._feed)
            model[path][self.COLUMN_MEMBER] = False
        else:
            category.append_feed(self._feed, False)
            model[path][self.COLUMN_MEMBER] = True

    def display_properties(self):
        self._title_default = self._feed.title
        loc, un, pw = self._feed.access_info
        self._url_default = loc
        self._un_default = un
        self._pw_default = pw
        self._refresh_default = self._feed.poll_frequency
        self._articles_default = self._feed.number_of_items_stored
        self._window.set_title(_("%s Properties") % self._feed.title)
        self._title.set_text(self._feed.title)
        self._location.set_text(self._feed.access_info[0])
        config = Config.get_instance()
        if self._feed.poll_frequency == Feed.Feed.DEFAULT:
            freq = config.poll_frequency
            fdefault = True
            self._refresh_keep = True
        else:
            freq = self._feed.poll_frequency
            fdefault = False
            self._refresh_keep = False
        self._refresh_spin.set_value(float(freq / 60))
        self._refresh_spin.set_sensitive(not fdefault)
        self._refresh_default_check.set_active(fdefault)
        if self._feed.number_of_items_stored == Feed.Feed.DEFAULT:
            nitems = config.number_of_items_stored
            nidefault = True
            self._articles_keep = True
        else:
            nitems = self._feed.number_of_items_stored
            nidefault = False
            self._articles_keep = False
        self._articles_spin.set_value(float(nitems))
        self._articles_spin.set_sensitive(not nidefault)
        self._articles_default_check.set_active(nidefault)

        if un:
            self._username.set_text(un)
            self._password.set_text(pw)

        model = self._categories_treeview.get_model()
        model.clear()
        fclist = FeedCategoryList.get_instance()
        for c in fclist.user_categories:
            iter = model.append()
            model.set(iter,
                      self.COLUMN_NAME, c.title,
                      self.COLUMN_MEMBER, self._feed in c.feeds,
                      self.COLUMN_OBJECT, c)

        self._previous_refresh_label.set_text(
            utils.format_date(time.gmtime(self._feed.last_poll)))
        next = self._feed.next_refresh
        if next is not None:
            self._next_refresh_label.set_text(
                utils.format_date(time.gmtime(next)))
            self._next_refresh_label.show()
        else:
            self._next_refresh_label.hide()

        self._display_feed_information(self._feed)
        self._restore_button.set_sensitive(False)

    def _display_feed_information(self, feed):
        title = utils.convert_entities(feed.channel_title)
        if len(title) == 0:
            title = feed.title
        if len(title) == 0:
            title = feed.channel_link
        title = title.strip()

        link = feed.channel_link.strip()
        if len(link) < 1: link = feed.location

        if len(title) > 0:
            self._feed_info_href.set_text(saxutils.escape(title))
            self._feed_info_href.set_url(link)
            self._feed_info_href.set_sensitive(len(link) > 0)
            self._feed_info_href.show()
        else:
            self._feed_info_href.hide()

        description = utils.convert_entities(feed.channel_description.strip())
        size = len(description)
        if size and description != title:
            description = read_text(description, size)
            self._feed_info_description.set_text(utils.convert_entities(description))
            self._feed_info_description.show()
        else:
            self._feed_info_description.hide()

        copyright = utils.convert_entities(feed.channel_copyright)
        self._feed_info_copyright.set_text(copyright)
        if len(copyright):
            self._feed_info_copyright.show()
        else:
            self._feed_info_copyright.hide()

    def restore_defaults(self):
        # FIXME: add frequency and number of items and the default flags
        self._feed.title = self._title_default
        self._feed.access_info = (
            self._url_default, self._un_default, self._pw_default)
        self._title.set_text(self._title_default)
        self._location.set_text(self._url_default)
        if self._un_default is not None:
            self._username.set_text(self._un_default)
        else:
            self._username.set_text("")
        if self._pw_default is not None:
            self._password.set_text(self._pw_default)
        else:
            self._password.set_text("")
        self._refresh_default_check.set_active(self._refresh_keep)
        if not self._refresh_keep:
            self._refresh_spin.set_value(float(self._refresh_default))
        self._articles_default_check.set_active(self._articles_keep)
        if not self._articles_keep:
            self._articles_spin.set_value(float(self._articles_default))
        self._restore_button.set_sensitive(False)

    def on_properties_title_entry_insert_text(self, *args):
        if self._initializing_window:
            return
        self._restore_button.set_sensitive(True)

    def on_properties_title_entry_delete_text(self, *args):
        if self._initializing_window:
            return
        self._restore_button.set_sensitive(True)

    def on_properties_title_entry_focus_out_event(self, widget, *args):
        self._feed.title = widget.get_text().strip()

    def on_properties_location_entry_insert_text(self, *args):
        if self._initializing_window:
            return
        self._restore_button.set_sensitive(True)

    def on_properties_location_entry_delete_text(self, *args):
        if self._initializing_window:
            return
        self._restore_button.set_sensitive(True)

    def on_properties_location_entry_focus_out_event(self, widget, *args):
        loc, username, pw = self._feed.access_info
        self._feed.access_info = (widget.get_text().strip(), username, pw)

    def on_properties_username_entry_insert_text(self, *args):
        if self._initializing_window:
            return
        self._restore_button.set_sensitive(True)

    def on_properties_username_entry_delete_text(self, *args):
        if self._initializing_window:
            return
        self._restore_button.set_sensitive(False)

    def on_properties_username_entry_focus_out_event(self, widget, *args):
        loc, username, pw = self._feed.access_info
        text = widget.get_text().strip()
        if text == "":
            text = None
        self._feed.access_info = (loc, text, pw)

    def on_properties_password_entry_insert_text(self, *args):
        if self._initializing_window:
            return
        self._restore_button.set_sensitive(True)

    def on_properties_password_entry_delete_text(self, *args):
        if self._initializing_window:
            return
        self._restore_button.set_sensitive(False)

    def on_properties_password_entry_focus_out_event(self, widget, *args):
        loc, username, pw = self._feed.access_info
        text = widget.get_text().strip()
        if text == "":
            text = None
        self._feed.access_info = (loc, username, text)

    def on_properties_reset_button_clicked(self, *args):
        self.restore_defaults()

    def on_properties_refresh_spin_focus_out_event(self, widget, *args):
        widget.update()
        value = widget.get_value_as_int()
        if value != self._feed.poll_frequency:
            self._feed.poll_frequency = value * 60

    def on_properties_articles_spin_focus_out_event(self, widget, *args):
        widget.update()
        value = widget.get_value_as_int()
        if value != self._feed.number_of_items_stored:
            self._feed.number_of_items_stored = value

    def on_properties_articles_spin_value_changed(self, widget, *args):
        if self._initializing_window:
            return
        self._restore_button.set_sensitive(True)

    def on_properties_refresh_spin_value_changed(self, widget, *args):
        if self._initializing_window:
            return
        self._restore_button.set_sensitive(True)

    def on_properties_keep_refresh_default_toggled(self, widget, *args):
        if self._initializing_window:
            return
        if widget.get_active():
            self._feed.poll_frequency = Feed.Feed.DEFAULT
            self._refresh_spin.set_sensitive(False)
            self._refresh_spin.set_value(float(Config.get_instance().poll_frequency / 60))
        else:
            self._feed.poll_frequency = self._refresh_spin.get_value_as_int() * 60
            self._refresh_spin.set_sensitive(True)
        self._restore_button.set_sensitive(True)

    def on_properties_keep_articles_default_toggled(self, widget, *args):
        if self._initializing_window:
            return
        if widget.get_active():
            self._feed.number_of_items_stored = Feed.Feed.DEFAULT
            self._articles_spin.set_value(float(Config.get_instance().number_of_items_stored))
            self._articles_spin.set_sensitive(False)
        else:
            self._feed.number_of_items_stored = self._articles_spin.get_value_as_int()
            self._articles_spin.set_sensitive(True)
        self._restore_button.set_sensitive(True)

    dialogs = {}
    def show_feed_properties(cls, parent, feed):
        class _dialogdeleter:
            def __init__(self, feed, hash):
                self.feed = feed
                self.hash = hash

            def __call__(self):
                del self.hash[feed]

        dialog = cls.dialogs.get(feed, None)
        if not dialog:
            dialog = cls(parent,feed,_dialogdeleter(feed, cls.dialogs))
            cls.dialogs[feed] = dialog
        dialog.show()
        return
    show_feed_properties = classmethod(show_feed_properties)

def read_text(fragment, chars):
    """Read chars cdata characters from html fragment fragment"""
    parser = SummaryParser.TitleImgParser()
    parser.feed(fragment)
    text = parser.get_text(chars)
    parser.close()
    return text



import unittest
import sys
import locale
sys.path.insert(0, "../src/lib")
import SummaryParser

try:
    import pmock
except ImportError:
    sys.exit("You need python-mock 0.3 to run this test. http://pmock.sf.net")


class TitleImgParserTest(unittest.TestCase):
    def setUp(self):
        self._parser = SummaryParser.TitleImgParser()

    def tearDown(self):
        self._parser.close()

    def testHTMLinTitle(self):
        fragment = "<p><a href=http://www.nongnu.org/straw>Straw</a>: <b>GNOME</b> <i>Desktop</i> Aggregator"
        self._parser.feed(fragment)
        self.assertEqual("Straw: GNOME Desktop Aggregator", self._parser.get_text())
        self._parser.close()
        fragment = "<span class=\"courtcase\">Lawrence</span>"
        self._parser.feed(fragment)
        self.assertEqual("Lawrence", self._parser.get_text())

    def testImgInHTML(self):
        fragment = "<p><img src='foo.jpg'/></p>"
        mock = pmock.Mock()
        mock.expects(pmock.at_least_once()).get_location().will(pmock.return_value('http://foo.com'))
        self._parser.set_feed(mock)
        self._parser.feed(fragment)
        mock.verify()
        self.assertEqual(["http://foo.com/foo.jpg"], self._parser.get_image_urls())
        fragment = "<a href='http://foo.com'><img src='/~jan/bar/foo/straw.png'></a>"
        self._parser.close()
        self._parser.feed(fragment)
        mock.verify()
        self.assertEqual(["http://foo.com/~jan/bar/foo/straw.png"], self._parser.get_image_urls())

    def testPlainTextInTitle(self):
        # Make sure plain text works too
        fragment = "Straw Rocks!"
        self._parser.feed(fragment)
        self.assertEqual("Straw Rocks!", self._parser.get_text())

    def testEntityInTitle(self):
        # http://bugzilla.gnome.org/show_bug.cgi?id=149924
        tdata =[("Joe &amp; Bob. One &lt; Two but &gt; 0","Joe & Bob. One < Two but > 0"),
                ("<b>Is<b> 2 &lt; <i>1<i>?\"foo\"","Is 2 < 1?\"foo\""),
                ("&lt;h2&gt;","<h2>")]

        for fragment,expected in tdata:
            self._parser.feed(fragment)
            self.assertEqual(expected, self._parser.get_text())
            self._parser.close()

    def testTitleLength(self):
        CHARS = 60
        # Excerpt from one of Jarno Virtannen's blog entries
        fragment = """
        I am not going to write much here for few weeks, because I
        <span class="bar">am taking</span> regular visits to a
        <a href="test.com">dentist</a>. I am not sure about the correct English term for my previous
        procedure, but I think it is called root canal treatment. The
        operation itself wasn't as painful as I was expecting (though not too
        pleasurable either) but I have to take antibiotics, which mess up my
        stomach and therefore I am not too energetic.
        """
        self._parser.feed(fragment)
        self.assertEqual(CHARS, len(self._parser.get_text(CHARS)))

    def testBrAndFontTagsInTitle(self):
        # http://bugzilla.gnome.org/show_bug.cgi?id=148105
        fragment="""
        <br><br><br><br><font color=444444 size=1>Shadow</font><br><br>
        """
        self._parser.feed(fragment)
        self.assertEqual("Shadow", self._parser.get_text())


def suite():
    return unittest.makeSuite(TitleParserTest, "test")

if __name__ == "__main__":
    unittest.main()



""" MainloopManager.py

"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """
Straw is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Straw is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """


import pygtk
pygtk.require("2.0")
import gtk
import gobject
import error

class MainloopManager:
    instance = None
    
    def __init__(self):
        self._idle_handlers = []
        self._timers = {}
        self._call_depth = 0

    def add_idle_handler(self, handler):
        oid = self._add_idle(handler)
        self._idle_handlers.append((oid, handler))

    def _add_idle(self, handler):
        oid = gtk.idle_add(handler)
        return oid

    def remove_idle_handler(self, handler):
        for oid, h in self._idle_handlers:
            if h == handler:
                gtk.idle_remove(oid)
                break
        else:
            error.log("no handler ", handler, " found!")

    def remove_idle_handler_by_id(self, oid):
        gtk.idle_remove(oid)

    def call_pending(self):
        if self._call_depth > 0:
            error.log("Trying re-entry! Not allowed.")
            return
        self._call_depth += 1
        try:
            for oid, handler in self._idle_handlers:
                self.remove_idle_handler_by_id(oid)
            while gtk.events_pending():
                gtk.main_iteration(False)

            for index in xrange(len(self._idle_handlers)):
                oid, handler = self._idle_handlers[index]
                new_id = self._add_idle(handler)
                self._idle_handlers[index] = (new_id, handler)
        finally:
            self._call_depth -= 1

    def set_repeating_timer(self, timeout, function, data = None):
        def f(*args):
            if not self._timers.has_key(function):
                return
            try:
                if len(args) > 0:
                    function(*args)
                else:
                    function()
            finally:
                if data is not None:
                    gobject.timeout_add(timeout, f, data)
                else:
                    gobject.timeout_add(timeout, f)
        if data is not None:
            gobject.timeout_add(timeout, f, data)
        else:
            gobject.timeout_add(timeout, f)
        self._timers[function] = True

    def end_repeating_timer(self, function):
        if self._timers.has_key(function):
            del self._timers[function]

    def get_instance(klass):
        if klass.instance is None:
            klass.instance = klass()
        return klass.instance
    get_instance = classmethod(get_instance)



""" Find.py

Module for searching items.
"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """ GNU General Public License

Straw is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Straw is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """


import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import gobject
import pango
import time
import Event
import utils
import FeedList
import MessageManager
from ItemList import ItemsView, ItemsPresenter
import MVP
import error

class Column:
    TITLE = 0
    FEED_TITLE = 1
    ITEM = 2
    BOLD = 3

class FindLimit(object):
    __slots__ = ('text', 'start', 'end')

    def __init__(self, text, start = None, end = None):
        self.text = text
        self.start = start
        self.end = end

    def is_subset(self, fl):
        if self.text.find(fl.text) == -1:
            return 0
        if fl.end and (self.end or self.end > fl.end):
            return 0
        if fl.start and (self.start or self.start < fl.start):
            return 0
        return 1

    def equals(self, fl):
        return (self.text == fl.text and
                self.end == fl.end and
                self.start == fl.start)

    def time_contains(self, item):
        rval = 1
        if item.pub_date: # pub_date hasn't been initialized in older versions of straw
            if self.start and self.start > item.pub_date:
                rval = 0
            if self.end and self.end < item.pub_date:
                rval = 0
        return rval

class Finder:
    def __init__(self):
        self._stack = list()
        self._feedlist = FeedList.get_instance()

    def find_matching(self, limit, items):
        if not len(items):
            for feed in self._feedlist:
                match = False
                for item in feed.items:
                    if limit.time_contains(item) and item.match(limit.text):
                        match = True
                        yield item
                if not match:
                    feed.unload_contents()
                while gtk.events_pending(): gtk.main_iteration(False)
            else:
                return
        else:
            error.log("ITEMS IS NOT NONE.!",items)
            # we expect this to be quicker than the above since the items are
            # already loaded in the memory.
            for item in items:
                if limit.time_contains(item) and item.match(limit.text):
                    yield item
        return


    def find(self, limit):
        if len(limit.text) < 1:
            del self._stack[:]
            return []
        items = []
        if len(self._stack):
            if limit.is_subset(self._stack[-1][0]):
                # the new search string is longer than the previous
                if len(self._stack[-1][1]):
                    # return existing items
                    items = self._stack[-1][1]
            else:
                # text was deleted
                length = len(self._stack)
                foundprev = 0
                while length > 0:
                    length -= 1
                    if limit.equals(self._stack[length][0]):
                        foundprev = 1
                        break
                    if limit.is_subset(self._stack[length][0]):
                        break
                self._stack = self._stack[:length+1]
                if foundprev or len(self._stack):
                    items = self._stack[-1][1]
        return (limit, items)

    def append_matches(self, limit, matches):
        lm = (limit,matches)
        self._stack.append(lm)
        return

class FindResultListView(ItemsView):
    """
    Widget: gtk.TreeView
    Model: Result Liststore
    Presenter: ItemListPresenter
    """
    def _initialize(self):
        ItemsView._initialize(self)
        self._widget.set_rules_hint(True)
        self._create_columns()
        selection =  self._widget.get_selection()
        selection.connect("changed", self._item_selection_changed)
        self._create_popup()

    def _model_set(self):
        self._widget.set_model(self._model)

    def _create_columns(self):
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Title', renderer, text=Column.TITLE,
                                    weight=Column.BOLD)
        self._widget.append_column(column)
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Feed', renderer, text=Column.FEED_TITLE,
                                    weight=Column.BOLD)
        self._widget.append_column(column)
        return

    def clear(self):
        self._widget.get_model().clear()

class FindResultPresenter(ItemsPresenter):
    def _initialize(self):
        self._connect()
        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING,
                              gobject.TYPE_PYOBJECT, gobject.TYPE_INT)
        self.model = model
        self._finder = Finder()
        self._interrupted = False
        self._matches=list()

    def _mark_item(self, item, index=None):
        if not index:
            index = self._matches.index(item)
        self._set_column_weight(item, index)

    def find(self, limit):
        self._clear_matches()

        limit_items = self._finder.find(limit)
        if not len(limit_items):
            return self._matches
        limit, items = limit_items
        for item in self._finder.find_matching(limit, items):
            if self._interrupted:
                self._interrupted = True
                break
            treeiter = self._model.append()
            weight = pango.WEIGHT_NORMAL
            if not item.seen:
                weight = pango.WEIGHT_BOLD
            self._matches.append(item)
            self._model.set(treeiter, Column.TITLE, item.title,
                            Column.FEED_TITLE, item.feed.title,
                            Column.ITEM, item,
                            Column.BOLD, weight)

        error.log("MATCHES: %d %s %s" %(len(self._matches),limit.text,items))
        self._finder.append_matches(limit, self._matches)
        return self._matches

    def stop(self, signal):
        error.log("stopping...")
        self._interrupted = True
        return

    def clear(self):
        self._interrupted = False
        self._model.clear()
        self._clear_matches()
        return

    def _clear_matches(self):
        for feed in dict.fromkeys((item.feed for item in self._matches)).keys():
            feed.unload_contents()
        del self._matches[:]

class FindView(MVP.WidgetView):
    SECS_DAY = 86400

    def _initialize(self):
        self._widget_tree = gtk.glade.get_widget_tree(self._widget)
        self._text_entry = self._widget_tree.get_widget("find_text_entry")

    def _on_find_text_entry_insert_text(self, *args):
        gobject.timeout_add(700, self._after_change)
        return

    def _on_find_text_entry_delete_text(self, *args):
        gobject.timeout_add(700, self._after_change)
        return

    def _on_find_start_time_limit_check_toggled(self, button, *args):
        state = button.get_active()
        start_date_widget = self._widget_tree.get_widget('find_start_date_edit')
        start_date_widget.set_sensitive(state)
        stime = self._determine_time(state, start_date_widget.get_time())
        self._presenter.start_date(stime)
        return

    def _on_find_end_time_limit_check_toggled(self, button, *args):
        state = button.get_active()
        end_date_widget = self._widget_tree.get_widget('find_end_date_edit')
        end_date_widget.set_sensitive(state)
        etime = self._determine_time(state, end_date_widget.get_time() + self.SECS_DAY)
        self._presenter.end_date(etime)
        return

    def _on_find_start_date_edit_date_changed(self, widget):
        """
        Precondition: start_date_toggle is toggled (ON) or else this signal
        won't be triggered in the first place
        """
        stime = self._determine_time(True, widget.get_time())
        self._presenter.start_date(stime)
        return

    def _on_find_end_date_edit_date_changed(self, widget):
        """
        Precondition: end_date_toggle is toggled (ON) or else this signal
        won't be triggered in the first place
        """
        etime = self._determine_time(True, widget.get_time() + self.SECS_DAY)
        self._presenter.end_date(etime)
        return

    def _after_change(self):
        newtext = self._text_entry.get_text()
        self._presenter.text_changed(newtext)
        return

    def _determine_time(self, togglestate, dtime):
        xtime = None
        if togglestate:
            xtime = time.gmtime(dtime)
        return xtime

    def _get_text(self): return self._text
    text = property(_get_text)

    def get_widget(self):
        return self._widget

    def clear(self):
        self._text_entry.delete_text(0,-1)
        return

class FindPresenter(MVP.BasicPresenter):
    def _initialize(self):
        self.initialize_slots(Event.FindInterruptSignal)
        widget_tree = gtk.glade.get_widget_tree(self._view.get_widget())
        self._find_result_pres = FindResultPresenter(view =
                                                     FindResultListView(widget_tree.get_widget("find_results_treeview")))
        self.signal_connect(Event.FindInterruptSignal,
                            self._find_result_pres.stop)
        self._resultscount = widget_tree.get_widget("find_results_count_display")
        self._rendering = False
        self._text = ''
        self._start_date = None
        self._end_date = None

    def start_date(self, stime):
        self._start_date = stime
        if stime and stime is not self._start_date:
            limit = FindLimit(self._text,
                              stime,
                              self._end_date)
            self._find_items(limit)
        return

    def end_date(self, etime):
        self._end_date = etime
        if etime and etime is not self._end_date:
            limit = FindLimit(self._text,
                              self._start_date,
                              etime)
            self._find_items(limit)
        return

    def text_changed(self, newtext):
        # terminate an existing result rendering in progress
        if self._rendering:
            self.emit_signal(Event.FindInterruptSignal(self))
        changed = newtext is not self._text
        if changed:
            self._text = newtext
            self._rendering = False
        if changed and not self._rendering:
            self._rendering = True
            limit = FindLimit(self._text,
                              self._start_date,
                              self._end_date)
            self._find_items(limit)
            self._rendering = False
        return

    def _find_items(self, limit):
        matches = self._find_result_pres.find(limit)
        nmatches = len(matches)
        if nmatches <= 0: self._find_result_pres.clear()
        self._resultscount.set_text(_("%d items found") % nmatches or 0)
        return

    def clear(self):
        self._find_result_pres.clear()
        self._view.clear()

    def _find_result_presenter(self): return self._find_result_pres
    item_list = property(_find_result_presenter)



#!/usr/bin/python

import unittest
import sys
import time

sys.path.insert(0, '../src/lib')
import LookupManager

class LookupTestCase(unittest.TestCase):
    def setUp(self):
        self.lm = LookupManager.get_instance()
        self.timeout = 60
        self.extraNames = (
            "art.gnome.org", "b9.com", "blog.neoteny.com",
            "diveintomark.org", "freshmeat.net", "home.comcast.net",
            "imperialviolet.org", "inessential.com", "kode-fu.com",
            "kt.zork.net", "lambda.weblogs.com",
            "morenews.blogspot.com", "mpt.phrasewise.com",
            "online.effbot.org", "patricklogan.blogspot.com",
            "people.enginesofcreation.ie", "polydistortion.net",
            "pragprog.com", "primates.ximian.com",
            "radio.weblogs.com", "rover.cs.northwestern.edu",
            "slashdot.org", "tbray.org", "toulouse.amber.org",
            "usefulinc.com", "weblog.infoworld.com",
            "weblogs.mozillazine.org", "wmf.editthispage.com",
            "ww.telent.net", "www.advogato.org", "www.amk.ca",
            "www.artima.com", "www.benhammersley.com",
            "www.blackbeltjones.com", "www.burtonini.com",
            "www.deadlybloodyserious.com", "www.debianplanet.net",
            "www.gnomedesktop.org", "www.hole.fi", "www.iki.fi",
            "www.inkstain.net", "www.lemonodor.com",
            "www.markokarppinen.com", "www.markwatson.com",
            "www.merrells.com", "www.mozillazine.org", "www.nat.org",
            "www.nongnu.org", "www.orbtech.com",
            "www.playbynumbers.com", "www.python.org",
            "www.rekaksois.com", "www.s2.org", "www.sidhe.org",
            "www.spacetoday.net", "www.sweetcode.org",
            "www.textism.com", "www.toolshed.com", "www.zanthan.com")

    def _callback(self, name, ip, data=None):
        self.response = (name, ip)

    def testGoodARecord(self):
        self.response = None
        self.lm.lookup('www.google.com', self._callback)
        now = time.time()
        while not self.response:
            assert time.time() - now < self.timeout, 'Lookup timed out'
            self.lm.poll(1)
        assert self.response[1] is not None

    def testBadARecord(self):
        self.response = None
        # this has, at least at some point, returned a cname even though
        # asked for an a record
        self.lm.lookup('www.morestuff4less.com', self._callback)
        now = time.time()
        while not self.response:
            assert time.time() - now < self.timeout, 'Lookup timed out'
            self.lm.poll(1)
        assert self.response[1] is not None

    def testEmptyDomain(self):
        self.response = None
        self.lm.lookup('', self._callback)
        now = time.time()
        while not self.response:
            assert time.time() - now < self.timeout, 'Lookup timed out'
            self.lm.poll(1)
        assert self.response[1] is None

    def testInvalidDomain(self):
        self.response = None
        self.lm.lookup('www.sdkfskdjjfsd.xcv', self._callback)
        now = time.time()
        while not self.response:
            assert time.time() - now < self.timeout, 'Lookup timed out'
            self.lm.poll(1)
        assert self.response[1] is None

    def _callback_multiple(self, name, ip, data=None):
        print "received %s for %s" % (name, ip)
        self.multiple_responses[name] = (name, ip)

    def testExtraNames(self):
        self.multiple_responses = {}
        for n in self.extraNames:
            self.multiple_responses[n] = None
            self.lm.lookup(n, self._callback_multiple)
        now = time.time()
        timeout = self.timeout * len(self.extraNames)
        while None in self.multiple_responses.values():
            assert time.time() - now < self.timeout, 'Lookup timed out'
            self.lm.poll(1)
        assert None not in self.multiple_responses.values()

def suite():
    suite = unittest.TestSuite()
    suite.addTest(LookupTestCase('testGoodARecord'))
    suite.addTest(LookupTestCase('testBadARecord'))
    suite.addTest(LookupTestCase('testEmptyDomain'))
    suite.addTest(LookupTestCase('testInvalidDomain'))
    suite.addTest(LookupTestCase('testExtraNames'))
    return suite

if __name__ == '__main__':
    unittest.main()

                  



""" FeedList.py

Provides a container for handling the feeds
"""
__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
__license__ = """GNU General Public License

This program is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA. """


import Event
import Feed
import Config
import locale
import utils

class FeedList(list, Event.SignalEmitter):
    def __init__(self, init_seq = []):
        Event.SignalEmitter.__init__(self)
        self.initialize_slots(Event.FeedsChangedSignal,
                              Event.FeedDeletedSignal,
                              Event.FeedCreatedSignal,
                              Event.AllItemsReadSignal,
                              Event.ItemReadSignal,
                              Event.ItemsAddedSignal,
                              Event.FeedPolledSignal,
                              Event.FeedStatusChangedSignal,
                              Event.FeedErrorStatusChangedSignal,
                              Event.FeedsImportedSignal,
                              Event.FeedDetailChangedSignal)
        self._loading = False

    def load_data(self):
        def _load(feeds, parent):
            for df in feeds:
                if isinstance(df, list):
                    #fc = straw.FeedCategory()
                    #fc.undump(df[0])
                    #self.append(parent, fc)
                    _load(df[1:], parent)
                else:
                    f = Feed.Feed.create_empty_feed()
                    f.undump(df)
                    self.append(parent, f)
        self._loading = True
        feeds = Config.get_instance().feeds
        if feeds is not None:
            _load(feeds, None)
        self._loading = False
        self.emit_signal(Event.FeedsChangedSignal(self))

    def _forward_signal(self, signal):
        self.emit_signal(signal)

    def connect_signals(self, ob):
        #ob.signal_connect(Event.FeedsChangedSignal,
        #                  self.feeds_changed, True)
        ob.signal_connect(Event.FeedDetailChangedSignal,
                          self.feed_detail_changed, True)

        # these signals are forwarded so that listeners who just want to
        # listen for a specific event regardless of what feed it came from can
        # just connect to this feedlist instead of connecting to the
        # individual feeds.
        ob.signal_connect(Event.AllItemsReadSignal, self._forward_signal)
        ob.signal_connect(Event.ItemReadSignal, self._forward_signal)
        ob.signal_connect(Event.ItemsAddedSignal, self._forward_signal)
        ob.signal_connect(Event.FeedPolledSignal, self._forward_signal)
        ob.signal_connect(Event.FeedStatusChangedSignal, self._forward_signal)
        ob.signal_connect(Event.FeedErrorStatusChangedSignal, self._forward_signal)

    def __setitem__(self, key, value):
        list.__setitem__(self, key, value)
        self.connect_signals(value)
        self.save_feeds_and_notify(True)

    def extend(self, parent, values, from_sub=False):
        list.extend(self, values)
        for f in values:
            f.parent = parent
            self.connect_signals(f)
        self.save_feeds_and_notify(
            True, signal=Event.FeedsImportedSignal(self, values, parent, from_sub))

    def append(self, parent, value):
        list.append(self, value)
        value.parent = parent
        self.connect_signals(value)
        self.save_feeds_and_notify(
            True, signal=Event.FeedCreatedSignal(self, value, parent))

    def insert(self, index, parent, value):
        list.insert(self, index, value)
        value.parent = parent
        self.connect_signals(value)
        self.save_feeds_and_notify(
            True, signal=Event.FeedCreatedSignal(self, value, parent, index))

    def index(self, feed):
        return list.index(self, feed)

    def reorder(self, move, delta):
        k = self[:]
        move = list(move)
        move.sort()
        if delta > 0:
            move.reverse()
        if move[0] == 0 and delta < 0 or move[-1] == (len(self) - 1) and delta > 0:
            return
        for m in move:
            k[m + delta], k[m] = k[m], k[m + delta]
        for i in range(len(k)):
            list.__setitem__(self, i, k[i])
        self.save_feeds_and_notify(True)

    def __delitem__(self, value):
        feed = self[value]
        list.__delitem__(self, value)
        feed.delete_all_items()
        self.save_feeds_and_notify(
            True, signal=Event.FeedDeletedSignal(self, feed))

    def save_feeds_and_notify(self, notify, signal=None):
        if not self._loading:
            config = Config.get_instance()
            config.feeds = [f.dump() for f in self]
            if notify:
                if not signal:
                    signal = Event.FeedsChangedSignal(self)
                self.emit_signal(signal)
        return

    def feeds_changed(self, signal, notify):
        self.save_feeds_and_notify(notify, signal)

    def feed_detail_changed(self, signal, notify):
        self.save_feeds_and_notify(notify, signal)

    def _sort_dsu(self, seq):
        aux_list = [(x.title, x) for x in seq]
        aux_list.sort(lambda a,b:locale.strcoll(a[0],b[0]))
        return [x[1] for x in aux_list]

    def sort(self, indices = None):
        if not indices or len(indices) == 1:
            self[:] = self._sort_dsu(self)
        else:
            items = self._sort_dsu(indices)
            for i,x in enumerate(items):
                list.__setitem__(self, indices[i], items[i])
        self.save_feeds_and_notify(True)

    def __hash__(self):
        h = 0
        for item in self:
            h ^= hash(item)
        return h

    def get_feed_with_id(self, id):
        for f in self.flatten_list():
            if f.id == id:
                return f
        return None

    def flatten_list(self, ob=None):
        if ob is None:
            ob = self
        l = []
        for o in ob:
            if isinstance(o, list):
                l = l + self.flatten_list(o)
            else:
                l.append(o)
        return l

feedlist_instance = None

def get_instance():
    global feedlist_instance
    if feedlist_instance is None:
        feedlist_instance = FeedList()
    return feedlist_instance




# Test Config.py

import unittest
import os
import sys

try:
    import pmock
except ImportError:
    sys.exit("You need python-mock (pmock, 0.3) to run this test. http://pmock.sf.net")

sys.path.insert(0, '../src/lib')
import Config

class ConfigTestCase(unittest.TestCase):
    def testPollFrequency(self):
        mock = pmock.Mock()
        poll_frequency = 18000
        mock.expects(pmock.once()).save_option(pmock.eq("/general/poll_frequency"),pmock.eq(poll_frequency))
        config = Config.Config(mock)
        config.set_poll_frequency(poll_frequency)
        mock.verify()
        self.assertEqual(poll_frequency, config.get_poll_frequency())

    def testLastPoll(self):
        mock = pmock.Mock()
        last_poll = 24600
        mock.expects(pmock.once()).save_option(pmock.eq("/general/last_poll"),pmock.eq(last_poll))
        config = Config.Config(mock)
        config.set_last_poll(last_poll)
        mock.verify()
        self.assertEqual(last_poll, config.get_last_poll())

    def testItemsStored(self):
        mock = pmock.Mock()
        stored = 150
        mock.expects(pmock.once()).save_option(pmock.eq("/general/number_of_items_stored"),pmock.eq(stored))
        config = Config.Config(mock)
        config.set_number_of_items_stored(stored)
        mock.verify()
        self.assertEqual(stored, config.get_number_of_items_stored())

    def testItemOrder(self):
        mock = pmock.Mock()
        order = True
        mock.expects(pmock.once()).save_option(pmock.eq("/general/item_order_newest"),pmock.eq(order))
        config = Config.Config(mock)
        config.set_item_order(order)
        mock.verify()
        self.assertEqual(order, config.get_item_order())

    def testOffline(self):
        mock = pmock.Mock()
        offline = False
        mock.expects(pmock.once()).save_option(pmock.eq("/general/offline"),pmock.eq(offline))
        config = Config.Config(mock)
        config.set_offline(offline)
        mock.verify()
        self.assertEqual(offline, config.get_offline())

    # feel free to add more tests here ...

def suite():
    suite = unittest.makeSuite(ConfigTestCase, 'test')
    return suite

if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""Straw: A feed aggregator for open source desktops.

Straw is a feed aggregator for open source desktops. Its goal is to be a
faster, easier, and more accessible way to read news and blogs than the
web browser. It uses the Python bindings to the GTK+ toolkit and supports RSS
and ATOM feeds.
"""

classifiers = """\
Development Status :: 4 - Beta
Environment :: X11 Applications :: GTK
Environment :: X11 Applications :: Gnome
Intended Audience :: End Users/Desktop
License :: OSI Approved :: GNU General Public License (GPL)
Operating System :: POSIX
Operating System :: POSIX :: Linux
Programming Language :: Python
Topic :: Desktop Environment
Topic :: Desktop Environment :: Gnome
Topic :: Internet :: WWW/HTTP
"""

import sys
import glob
import os.path

strawenv = {'APPNAME': 'Straw',
            'VERSION': '0.27',
            'PYTHON':  '2.4',
            'PYGTK': (2,8,0)}

if sys.version < strawenv['PYTHON']:
    sys.exit('Error: %s or newer is required. Current version:\n %s'
             % (strawenv['PYTHON'],sys.version))

doclines = __doc__.split("\n")

def modules_check():
    '''Check if necessary modules is installed.
    The function is executed by distutils (by the install command).'''
    # Set This For Machines with No DISPLAY
    nogtk = 0

    import imp
    # Check for PyGTK version
    try:
        import pygtk
        pygtk.require('2.0')
        imp.find_module('gtk')
        import gtk
        if gtk.pygtk_version < strawenv['PYGTK']:
            raise
    except AssertionError:
        pass # We ignore this because gtk must be present to build"
    except RuntimeError, msg:
        if os.environ.get ('DISPLAY', '') != '':
            etype, value, tb = sys.exc_info ()
            traceback.print_exception (etype, value, tb)

            sys.exit('Error: unexpected runtime error checking for gtk module')
	nogtk = 1
        pass # No DISPLAY set, pass
    except:
        sys.exit('Error: PyGTK-%d.%d.%d or newer is required.' % strawenv['PYGTK'])

    # Check other module requirements.
    # In pygtk versions prior to 2.10, gtkhtml2 is included in
    # the 'gnome' module. But as of 2.10, the gtkhtml2 module was
    # removed in 'gnome' and added in 'gnome-extras' module.
    # The egg.trayicon module is also included in g-p-e, so no need to check here.
    mod_list = [
        ('gnome', 'python-gnome2', 0),
        ('gtkhtml2', "python-gnome2-extras", 0),
        ('gconf', "python-gnome2", 1),
        ('bsddb.db', 'python-bsddb', 0),
        ('dbus', 'python-dbus', 0)]

    for m, w, x in mod_list:
	if nogtk == 1 and m == 'gtkhtml2':
	    pass;
	else:
            try:
                if not x: exec('import %s' % m)
                else: imp.find_module(m)
            except ImportError, msg:
                sys.exit("Error: '%s' module in package '%s' is required to install %s" \
                      % (m,w,strawenv['APPNAME']))

    # Check for dbus version
    if getattr(dbus, 'version', None) is None:
        print "Warning: Recent versions of  DBus is needed to run Straw"

    # gtk.glade needs special care (ugly...)
    path = imp.find_module('gtk')
    if not os.path.exists(path[1] + '/glade.so'):
        sys.exit('Error: %s module is required to install %s' \
                 % ("PyGTK's glade", strawenv['APPNAME']))

    # Check for ADNS
    try:
        import adns, ADNS
    except ImportError:
        print 'Warning: ADNS Python module not found, will continue without.'

def translations():
    '''Build mo-po mapping from po directory'''
    trans = []
    dest = 'share/locale/%s/LC_MESSAGES/%s.mo'
    for po in glob.glob('po/*.po'):
        lang = os.path.splitext(os.path.basename(po))[0]
        trans.append((dest % (lang , strawenv['APPNAME'].lower()), po))
    return trans

def translation_files():
    '''Files for translation...'''
    potfile = './po/POTFILES.in'
    if not os.path.exists(potfile):
        sys.exit("No such file, '%s'. This file should've been built \
        automatically. You can run \"intltool-update --pot\" to create it manually.\
        We also encourage you to file a bug report against Straw." % potfile)
    try:
        f = open(potfile)
        files = []
        for line in f:
            # ignore comments and newline
            if not line.startswith('#') or not line.startswith('\n'):
                files.append(line.strip())
    finally:
        f.close()
    return files

def data_files():
    '''Build list of data files to be installed'''
    images = glob.glob('images/*.png')
    misc = [
        'data/default_subscriptions.opml',
        'data/straw.css',
        'glade/straw.glade'
        ]
    files = [
        ('share/pixmaps', ['images/straw.png']),
        ('share/straw', images + misc)]
    return files

# Let distutils do the work
from tools.straw_distutils import setup
setup(name             = strawenv['APPNAME'],
      version          = strawenv['VERSION'],
      description      = doclines[0],
      long_description = "\n".join(doclines[2:]),
      author           = 'Juri Pakaste',
      author_email     = 'juri@iki.fi',
      maintainer       = 'Jan Alonzo',
      maintainer_email = 'jmalonzo@unpluggable.com',
      url              = 'http://www.gnome.org/projects/straw/',
      license          = 'GPL',
      data_files       = data_files(),
      pot_file         = 'po/straw.pot',
      translations     = translations(),
      modules_check    = modules_check,
      msg_sources      = translation_files(),
      desktop_file     = ['straw.desktop.in'],
      constants        = [('constants.py.in', strawenv)],
      scripts          = ['src/straw'],
      packages         = ['straw'],
      package_dir      = {'straw' : 'src/lib'},
      config_files     = [('gconf/schemas',['data/straw.schemas'],
                         'with-gconf-schema-file-dir')],
      classifiers      = filter(None, classifiers.split("\n")))
